@@ -81,10 +81,8 @@ static void free_kv(key_value_t * kv);
  * 997, 1009, 1021, 1031
  */
 #define HASH_TABLE_SIZE 127
-#define HASH_INDEX_BITS 7
-#define HASH_INDEX_MASK ((1 << HASH_INDEX_BITS) - 1)
 
-#define HASH_INDEX(string) (hash_value(string) & HASH_INDEX_MASK)
+#define HASH_INDEX(string) (hash_value(string) % HASH_TABLE_SIZE)
 
 static key_value_t * * hash_table = NULL;
 
@@ -317,13 +315,15 @@ static void compress_escapes(char * string) {
 
 
 static inline void no_value_msg(char * key) {
-	engine_user_message(NULL,NULL,"Parse error on line %d in file %s.  Key %s does not have a value assigned to it.\n",
+	engine_user_message(NULL, NULL,
+			    _("Parse error on line %d in file %s.  Key \"%s\" does not have a value assigned to it.\n"),
 			    line_num, config_file_name, key);
 }
 
 
 static inline void bad_value_msg(char * key) {
-	engine_user_message(NULL,NULL,"Parse error on line %d in file %s.  %s not a valid value.\n",
+	engine_user_message(NULL, NULL,
+			    _("Parse error on line %d in file %s.  %s not a valid value.\n"),
 			    line_num, config_file_name, key);
 }
 
@@ -371,7 +371,8 @@ static char * find_string_end(char * p, char * delimiters) {
 				p++;
 
 			} else {
-				engine_user_message(NULL,NULL,"Parse error on line %d in file %s.  String does not have a closing single quote.\n",
+				engine_user_message(NULL, NULL,
+						    _("Parse error on line %d in file %s.  String does not have a closing single quote.\n"),
 						    line_num, config_file_name);
 			}
 			break;
@@ -394,7 +395,8 @@ static char * find_string_end(char * p, char * delimiters) {
 				p++;
 
 			} else {
-				engine_user_message(NULL,NULL,"Parse error on line %d in file %s.  String does not have a closing double quote.\n",
+				engine_user_message(NULL, NULL,
+						    _("Parse error on line %d in file %s.  String does not have a closing double quote.\n"),
 						    line_num, config_file_name);
 			}
 			break;
@@ -566,7 +568,7 @@ static key_value_t * parse_section(char * * pp, char * key) {
 					if (*p != '\0') {
 						kv_child = parse_value(&p, key);
 
-						if (kv != NULL) {
+						if (kv_child != NULL) {
 							kv_child->next = kv->value.section;
 							kv->value.section = kv_child;
 						}
@@ -588,7 +590,8 @@ static key_value_t * parse_section(char * * pp, char * key) {
 			p++;
 
 		} else {
-			engine_user_message(NULL,NULL,"Parse error on line %d in file %s.  Section does not have a closing curly brace (}).\n",
+			engine_user_message(NULL, NULL,
+					    _("Parse error on line %d in file %s.  Section does not have a closing curly brace (}).\n"),
 					    line_num, config_file_name);
 		}
 
@@ -709,7 +712,8 @@ static key_value_t * parse_array(char * * pp, char * key) {
 					p++;
 
 				} else {
-					engine_user_message(NULL,NULL,"Parse error on line %d in file %s.  Section does not have a closing square bracket (]).\n",
+					engine_user_message(NULL ,NULL,
+							    _("Parse error on line %d in file %s.  Section does not have a closing square bracket (]).\n"),
 							    line_num, config_file_name);
 				}
 
@@ -779,7 +783,8 @@ static key_value_t * parse_value(char * * pp, char * key) {
 
 	case '}':
 	case ']':
-		engine_user_message(NULL,NULL,"Parse error on line %d in file %s.  Unexpected '%c'.\n",
+		engine_user_message(NULL, NULL,
+				    _("Parse error on line %d in file %s.  Unexpected '%c'.\n"),
 				    line_num, config_file_name, *(*pp));
 		break;
 
@@ -980,8 +985,8 @@ int evms_get_config(char * file_name) {
 	if (hash_table != NULL) {
 		if (!(strcmp(config_file_name, DEFAULT_CONFIG_FILE_NAME) == 0)) {
 			engine_user_message(NULL,NULL,
-					    "Unable to get the configuration from file %s.  "
-					    "The configuration has already been read from file %s.\n",
+					    _("Unable to get the configuration from file %s.  "
+					      "The configuration has already been read from file %s.\n"),
 					    file_name, config_file_name);
 
 			LOG_PROC_EXIT_INT(EINVAL);
