@@ -44,6 +44,7 @@
 #include "cluster.h"
 #include "dmonmsg.h"
 #include <ldm_funcs.h>
+#include "activation.h"
 
 
 boolean discover_in_progress = FALSE;
@@ -173,10 +174,11 @@ int ensure_dev_node(char * name, u_int32_t major, u_int32_t minor) {
 
 			case EEXIST:
 				/* Dev node exists but has the wrong major:minor.  Fix it. */
-				LOG_DEBUG("The device node for volume \"%s\" has the wrong major:minor.  I'm fixing it to %d:%d.\n", name, major, minor);
+				LOG_DEBUG("The device node \"%s\" has the wrong major:minor.  I'm fixing it to %d:%d.\n", name, major, minor);
 
 				rc = unlink(name);
 				if (rc == 0) {
+					LOG_DEBUG("Make dev node for \"%s\".\n", name);
 					rc = mknod(name, S_IFBLK | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP, devt);
 
 					if (rc != 0) {
@@ -189,6 +191,8 @@ int ensure_dev_node(char * name, u_int32_t major, u_int32_t minor) {
 				break;
 
 			case ENOENT:
+				LOG_DEBUG("Device node \"%s\" does not exist.", name);
+
 				strcpy(dir_name, name);
 				*(strrchr(dir_name, '/')) = '\0';
 
@@ -197,6 +201,7 @@ int ensure_dev_node(char * name, u_int32_t major, u_int32_t minor) {
 							       S_IROTH | S_IXOTH));
 
 				if (rc == 0) {
+					LOG_DEBUG("Make dev node for \"%s\".\n", name);
 					rc = mknod(name, S_IFBLK | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP, devt);
 
 					if (rc != 0) {
@@ -211,11 +216,10 @@ int ensure_dev_node(char * name, u_int32_t major, u_int32_t minor) {
 
 			default:
 				LOG_WARNING("hasa_dev_node() for \"%s\", major %d, minor %d failed with unexpected error %d: %s.\n", name, major, minor, rc, strerror(rc));
-
 		}
 
 	} else {
-		LOG_ERROR("Major:minor %d:%d is not valid.\n", major, minor);
+		LOG_DEBUG("Major:minor %d:%d is not valid.\n", major, minor);
 		rc = EINVAL;
 	}
 
@@ -351,6 +355,24 @@ char * status_dots(int * status_count) {
 }
 
 
+static int discover_replace_objects(list_anchor_t object_list) {
+
+	int rc = 0;
+	STATIC_LIST_DECL(result_object_list);
+
+	LOG_PROC_ENTRY();
+
+	if (replace_plugin != NULL) {
+		rc = replace_plugin->functions.plugin->discover(object_list, &result_object_list, FALSE);
+		delete_all_elements(object_list);
+		merge_lists(object_list, &result_object_list, NULL, NULL);
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
 /******************************************************************************
  *
  * Functions for discovering logical disks
@@ -378,19 +400,21 @@ static int discover_logical_disks(list_anchor_t object_list, boolean report_stat
 			storage_object_t * obj;
 
 			if (report_status) {
-				status_message("Discovering disks%s\n", status_dots(&status_count));
+				status_message(_("Discovering disks%s\n"), status_dots(&status_count));
 			}
 
 			LOG_DEBUG("Calling discover() in %s.\n", pPlugRec->short_name);
-			rc = pPlugRec->functions.plugin->discover(object_list, &result_object_list, TRUE);
-			LOG_DEBUG("Return code from discover() is %d.\n", rc);
+			rc = pPlugRec->functions.plugin->discover(object_list, &result_object_list, FALSE);
+			LOG_DEBUG("Return code from discover() is %d: %s.\n", rc, evms_strerror(rc));
 
 			/* Remove any corrupt objects from the list. */
 			remove_corrupt_objects(&result_object_list);
 
 			/* Make sure we have a dev node for each object. */
 			LIST_FOR_EACH(&result_object_list, iter, obj) {
-				make_object_dev_node(obj);
+				if (obj->flags & SOFLAG_ACTIVE) {
+					make_object_dev_node(obj);
+				}
 			}
 
 			/* Remove any remaining objects from the source list. */
@@ -400,7 +424,9 @@ static int discover_logical_disks(list_anchor_t object_list, boolean report_stat
 			 * Put the objects from the result list onto the source
 			 * list.
 			 */
-			merge_lists(object_list, &result_object_list, NULL);
+			merge_lists(object_list, &result_object_list, NULL, NULL);
+
+			discover_replace_objects(object_list);
 		}
 	}
 
@@ -449,7 +475,7 @@ static int discover_segments(list_anchor_t object_list, boolean report_status) {
 				list_element_t iter2;
 
 				if (report_status) {
-					status_message("Discovering segments%s\n", status_dots(&status_count));
+					status_message(_("Discovering segments%s\n"), status_dots(&status_count));
 				}
 
 				LOG_DEBUG("Calling discover() in %s.  last_pass is %s.\n", pPlugRec->short_name, (last_pass) ? "TRUE" : "FALSE");
@@ -461,7 +487,9 @@ static int discover_segments(list_anchor_t object_list, boolean report_status) {
 
 				/* Make sure we have a dev node for each object. */
 				LIST_FOR_EACH(&result_object_list, iter2, obj) {
-					make_object_dev_node(obj);
+					if (obj->flags & SOFLAG_ACTIVE) {
+						make_object_dev_node(obj);
+					}
 				}
 
 				propigate_cluster_info(&result_object_list);
@@ -494,7 +522,9 @@ static int discover_segments(list_anchor_t object_list, boolean report_status) {
 				 * Put the objects from the result list onto the source
 				 * list.
 				 */
-				merge_lists(object_list, &result_object_list, NULL);
+				merge_lists(object_list, &result_object_list, NULL, NULL);
+
+				discover_replace_objects(object_list);
 			}
 		}
 
@@ -518,7 +548,7 @@ static int discover_segments(list_anchor_t object_list, boolean report_status) {
 	 * Put the unclaimed objects back on the object_list so that they
 	 * can be examined by the next layer of discovery.
 	 */
-	merge_lists(object_list, &unclaimed_object_list, NULL);
+	merge_lists(object_list, &unclaimed_object_list, NULL, NULL);
 
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
@@ -565,7 +595,7 @@ static int discover_regions(list_anchor_t object_list, boolean report_status) {
 				list_element_t iter2;
 
 				if (report_status) {
-					status_message("Discovering regions%s\n", status_dots(&status_count));
+					status_message(_("Discovering regions%s\n"), status_dots(&status_count));
 				}
 
 				LOG_DEBUG("Calling discover() in %s.  last_pass is %s.\n", pPlugRec->short_name, (last_pass) ? "TRUE" : "FALSE");
@@ -577,7 +607,9 @@ static int discover_regions(list_anchor_t object_list, boolean report_status) {
 
 				/* Make sure we have a dev node for each object. */
 				LIST_FOR_EACH(&result_object_list, iter2, obj) {
-					make_object_dev_node(obj);
+					if (obj->flags & SOFLAG_ACTIVE) {
+						make_object_dev_node(obj);
+					}
 				}
 
 				propigate_cluster_info(&result_object_list);
@@ -611,7 +643,9 @@ static int discover_regions(list_anchor_t object_list, boolean report_status) {
 				 * list.  AppendList deletes them from the
 				 * result_object_list.
 				 */
-				merge_lists(object_list, &result_object_list, NULL);
+				merge_lists(object_list, &result_object_list, NULL, NULL);
+
+				discover_replace_objects(object_list);
 			}
 		}
 
@@ -635,7 +669,7 @@ static int discover_regions(list_anchor_t object_list, boolean report_status) {
 	 * Put the unclaimed objects back on the object_list so that they
 	 * can be examined by the next layer of discovery.
 	 */
-	merge_lists(object_list, &unclaimed_object_list, NULL);
+	merge_lists(object_list, &unclaimed_object_list, NULL, NULL);
 
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
@@ -854,11 +888,6 @@ int make_volume_dev_node(logical_volume_t * vol) {
 
 		if (rc == 0) {
 			memcpy(vol->dev_node, vol->name, sizeof(vol->name));
-			vol->flags &= ~VOLFLAG_NEEDS_ACTIVATE;
-
-			if (vol->object->flags & SOFLAG_ACTIVE) {
-				vol->flags |= VOLFLAG_ACTIVE;
-			}
 
 		} else {
 			memset(vol->dev_node, '\0', sizeof(vol->dev_node));
@@ -901,10 +930,6 @@ static int get_volume_dev_node_and_fsim(logical_volume_t * vol) {
 		}
 	}
 
-	if (!(vol->flags & VOLFLAG_ACTIVE)) {
-		vol->flags |= VOLFLAG_NEEDS_ACTIVATE;
-	}
-
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
 }
@@ -1020,7 +1045,7 @@ static int validate_feature_header(evms_feature_header_t * fh) {
 
 		old_crc = DISK_TO_CPU32(fh->crc);
 		fh->crc = 0;
-		new_crc = calculate_CRC(0xFFFFFFFFL, fh, sizeof(evms_feature_header_t));
+		new_crc = evms_calculate_crc(EVMS_INITIAL_CRC, fh, sizeof(evms_feature_header_t));
 
 		if ((new_crc == old_crc) || (old_crc == EVMS_MAGIC_CRC)) {
 
@@ -1280,7 +1305,7 @@ static void get_feature_header(storage_object_t * obj) {
 						 * don't use any of the feature headers.
 						 */
 						engine_user_message(NULL, NULL,
-								    "Error code %d (%s) when reading the secondary copy of feature header on object %s.  Ignoring good primary feature header.\n",
+								    _("Error code %d (%s) when reading the secondary copy of feature header on object %s.  Ignoring good primary feature header.\n"),
 								    secondary_rc, strerror(secondary_rc), obj->name);
 						engine_free(primary_fh);
 						break;
@@ -1324,7 +1349,8 @@ static void get_feature_header(storage_object_t * obj) {
 						 * don't use any of the feature headers.
 						 */
 						LOG_WARNING("Primary feature header not found on object %s.\n", obj->name);
-						engine_user_message(NULL, NULL, "Error code %d (%s) when reading the secondary copy of feature header on object %s.\n",
+						engine_user_message(NULL, NULL,
+								    _("Error code %d (%s) when reading the secondary copy of feature header on object %s.\n"),
 								    secondary_rc, strerror(secondary_rc), obj->name);
 						break;
 				}
@@ -1336,12 +1362,14 @@ static void get_feature_header(storage_object_t * obj) {
 				 * header.  If we get an unexpected error we don't use any of
 				 * the feature headers.
 				 */
-				engine_user_message(NULL, NULL, "Error code %d (%s) when reading the primary copy of feature header on object %s.\n",
+				engine_user_message(NULL, NULL,
+						    _("Error code %d (%s) when reading the primary copy of feature header on object %s.\n"),
 						    primary_rc, strerror(primary_rc), obj->name);
 
 				switch (secondary_rc) {
 					case 0:
-						engine_user_message(NULL, NULL, "Ignoring good secondary feature header.\n");
+						engine_user_message(NULL, NULL,
+								    _("Ignoring good secondary feature header.\n"));
 						engine_free(secondary_fh);
 						break;
 
@@ -1355,7 +1383,8 @@ static void get_feature_header(storage_object_t * obj) {
 						 * We got an unexpected error when reading the
 						 * secondary feature header.
 						 */
-						engine_user_message(NULL, NULL, "Error code %d (%s) when reading the secondary copy of feature header on object %s.\n",
+						engine_user_message(NULL, NULL,
+								    _("Error code %d (%s) when reading the secondary copy of feature header on object %s.\n"),
 								    secondary_rc, strerror(secondary_rc), obj->name);
 						break;
 				}
@@ -1411,7 +1440,7 @@ static int discover_objects_by_plugin(list_anchor_t object_list,
 	    obj = first_thing(object_list, &iter)) {
 
 		plugin_record_t * feature;
-		list_element_t el = NULL;;
+		list_element_t el = NULL;
 
 		remove_element(iter);
 
@@ -1467,7 +1496,8 @@ static int discover_objects_by_plugin(list_anchor_t object_list,
 							storage_object_t * tmp_obj;
 
 							if (report_status) {
-								status_message("Discovering EVMS objects%s\n", status_dots(&status_count));
+								status_message(_("Discovering EVMS objects%s\n"),
+									       status_dots(&status_count));
 							}
 							rc = feature->functions.plugin->discover(&plugin_object_list, &new_object_list, TRUE);
 
@@ -1479,7 +1509,9 @@ static int discover_objects_by_plugin(list_anchor_t object_list,
 
 							/* Make sure we have a dev node for each object. */
 							LIST_FOR_EACH(&new_object_list, tmp_iter, tmp_obj) {
-								make_object_dev_node(tmp_obj);
+								if (tmp_obj->flags & SOFLAG_ACTIVE) {
+									make_object_dev_node(tmp_obj);
+								}
 							}
 
 							propigate_cluster_info(&new_object_list);
@@ -1491,7 +1523,10 @@ static int discover_objects_by_plugin(list_anchor_t object_list,
 							if (rc == 0) {
 								merge_lists(resulting_object_list,
 									    &new_object_list,
+									    NULL,
 									    NULL);
+
+								discover_replace_objects(resulting_object_list);
 							}
 						}
 					}
@@ -1768,20 +1803,48 @@ static int process_object(storage_object_t   * obj,
 
 	} else {
 		/*
-		 * The object has a feature header.  If the feature header has an
-		 * object_depth of 1 and a feature_id of 0, then it is a feature
-		 * header for making an EVMS volume.
+		 * The object has a feature header.  If the feature header has
+		 * an object_depth of 1 and a feature_id of 0, then it is a
+		 * feature header for making an EVMS volume.
 		 */
 
 		if ((obj->feature_header->object_depth == 1) &&
 		    (obj->feature_header->feature_id == EVMS_VOLUME_FEATURE_ID)) {
 
 			if (!(obj->feature_header->flags & EVMS_VOLUME_DATA_OBJECT)) {
+				sector_count_t usable_size = obj->size - EVMS_FEATURE_HEADER_SECTORS * 2;
+				usable_size = round_down_to_hard_sector(usable_size, obj);
+				
 				/* Don't let errors stop the discovery process. */
 				make_evms_volume_for_object(obj,
 							    obj->feature_header->volume_name,
 							    obj->feature_header->volume_serial_number);
-				get_volume_dev_node_and_fsim(obj->volume);
+				if (obj->volume != NULL) {
+					get_volume_dev_node_and_fsim(obj->volume);
+
+					/*
+					 * If the device for the volume is
+					 * bigger than the device for its object
+					 * and the file system on the device
+					 * fits within the object size resize
+					 * the volume to the object size.
+					 */
+					if (obj->volume->vol_size < usable_size) {
+						if (obj->volume->fs_size <= usable_size) {
+							obj->volume->vol_size = usable_size;
+							obj->volume->flags |= VOLFLAG_NEEDS_ACTIVATE;
+
+						} else {
+							LOG_SERIOUS("The DM device for volume %s is of size %"PRIu64" which is "
+								    "bigger than the DM device for the volume's object %s of size %"PRIu64".  "
+								    "I'm deactivating the volume.\n",
+								    obj->volume->name, obj->volume->vol_size,
+								    obj->name, obj->size);
+
+							obj->volume->flags |= VOLFLAG_NEEDS_DEACTIVATE;
+						}
+					}
+				}
 			}
 
 		} else {
@@ -1834,7 +1897,7 @@ static void remove_non_data_objects(list_anchor_t list) {
 /*
  * Scan a list, removing any object that has stop data on it.
  */
-static evms_feature_header_t stop_data;
+static evms_feature_header_t stop_data_fh;
 
 static void remove_stop_data_objects(list_anchor_t list) {
 
@@ -1855,15 +1918,15 @@ static void remove_stop_data_objects(list_anchor_t list) {
 		if (obj->data_type == DATA_TYPE) {
 
 			/* Check for stop data at the end of the object. */
-			rc = obj->plugin->functions.plugin->read(obj, obj->size - 1, 1, &stop_data);
+			rc = obj->plugin->functions.plugin->read(obj, obj->size - 1, 1, &stop_data_fh);
 
 			if (rc == 0) {
-				if (validate_feature_header(&stop_data) == 0) {
+				if (validate_feature_header(&stop_data_fh) == 0) {
 					/*
 					 * validate_feature_header() converts the in-memory
 					 * feature header to CPU endian format.
 					 */
-					if (stop_data.flags & EVMS_VOLUME_DATA_STOP) {
+					if (stop_data_fh.flags & EVMS_VOLUME_DATA_STOP) {
 						LOG_DEBUG("Object %s has stop data on it.\n", obj->name);
 						obj->flags |= SOFLAG_HAS_STOP_DATA;
 						delete_element(iter1);
@@ -2011,7 +2074,8 @@ static int discover_associative_features(list_anchor_t object_list, boolean repo
 				storage_object_t * tmp_obj;
 
 				if (report_status) {
-					status_message("Discovering EVMS associative objects%s\n", status_dots(&status_count));
+					status_message(_("Discovering EVMS associative objects%s\n"),
+						       status_dots(&status_count));
 				}
 				rc = feature->functions.plugin->discover(&object_list, &new_object_list, TRUE);
 
@@ -2021,7 +2085,9 @@ static int discover_associative_features(list_anchor_t object_list, boolean repo
 				remove_corrupt_objects(&new_object_list);
 
 				LIST_FOR_EACH(&new_object_list, iter, tmp_obj) {
-					make_object_dev_node(tmp_obj);
+					if (tmp_obj->flags & SOFLAG_ACTIVE) {
+						make_object_dev_node(tmp_obj);
+					}
 				}
 
 				propigate_cluster_info(&new_object_list);
@@ -2078,7 +2144,8 @@ static int discover_associative_features(list_anchor_t object_list, boolean repo
 
 
 int compare_objects(void * thing1,
-		    void * thing2) {
+		    void * thing2,
+		    void * user_data) {
 
 	int result = 0;
 
@@ -2147,17 +2214,21 @@ int compare_objects(void * thing1,
 						child1 = first_thing(obj1->child_objects, NULL);
 						child2 = first_thing(obj2->child_objects, NULL);
 
-						result = strcmp(child1->name, child2->name);
+						if (child1 != NULL && child2 != NULL) {
+							result = strcmp(child1->name, child2->name);
 
-						if (result == 0) {
-							/* Segments are sorted by starting LBA. */
-							if (obj1->start < obj2->start) {
-								result = -1;
-							} else if (obj1->start > obj2->start) {
-								result = 1;
-							} else {
-								result = 0;
+							if (result == 0) {
+								/* Segments are sorted by starting LBA. */
+								if (obj1->start < obj2->start) {
+									result = -1;
+								} else if (obj1->start > obj2->start) {
+									result = 1;
+								} else {
+									result = 0;
+								}
 							}
+						} else {
+							result = strcmp(obj1->name, obj2->name);
 						}
 					}
 					break;
@@ -2181,7 +2252,8 @@ int compare_objects(void * thing1,
 
 
 int compare_containers(void * thing1,
-		       void * thing2) {
+		       void * thing2,
+		       void * user_data) {
 
 	int result = 0;
 
@@ -2204,7 +2276,8 @@ int compare_containers(void * thing1,
 
 
 int compare_volumes(void * thing1,
-		    void * thing2) {
+		    void * thing2,
+		    void * user_data) {
 
 	int result = 0;
 
@@ -2613,12 +2686,12 @@ static void cleanup_dev_evms_tree(void) {
 	
 	sync_volumes();
 
-	status_message("Cleaning up the /dev/evms tree%s\n", status_dots(&status_count));
+	status_message(_("Cleaning up the /dev/evms tree%s\n"), status_dots(&status_count));
 
 	/* Get an array of all the block dev nodes in the entire /dev/evms tree. */
 	get_dev_node_names(EVMS_DEV_NODE_DIR, &dev_nodes, &num_nodes);
 
-	status_message("Cleaning up the /dev/evms tree%s\n", status_dots(&status_count));
+	status_message(_("Cleaning up the /dev/evms tree%s\n"), status_dots(&status_count));
 
 	/*
 	 * dev_nodes contains a sorted array of nodes in the /dev/evms tree.
@@ -2626,20 +2699,22 @@ static void cleanup_dev_evms_tree(void) {
 	 * about.
 	 */
 	LIST_FOR_EACH(&volumes_list, iter, vol) {
-		remove_name(vol->name, dev_nodes, &num_nodes);
+		if (vol->flags & VOLFLAG_ACTIVE) {
+			remove_name(vol->name, dev_nodes, &num_nodes);
+		}
 	}
 
 	for (i = 0; lists[i] != NULL; i++) {
-		status_message("Cleaning up the /dev/evms tree%s\n", status_dots(&status_count));
+		status_message(_("Cleaning up the /dev/evms tree%s\n"), status_dots(&status_count));
 		LIST_FOR_EACH(lists[i], iter, obj) {
-			if (obj->data_type == DATA_TYPE) {
+			if (obj->flags & SOFLAG_ACTIVE) {
 				strcpy(&obj_name_buf[EVMS_OBJECT_NODE_PATH_LEN], obj->name);
 				ensure_dev_node(obj_name_buf, obj->dev_major, obj->dev_minor);
 				remove_name(obj_name_buf, dev_nodes, &num_nodes);
 			}
 		}
 	}
-	status_message("Cleaning up the /dev/evms tree%s\n", status_dots(&status_count));
+	status_message(_("Cleaning up the /dev/evms tree%s\n"), status_dots(&status_count));
 
 	/*
 	 * All of the known nodes have been removed from the list.  What is left in
@@ -2657,7 +2732,7 @@ static void cleanup_dev_evms_tree(void) {
 
 	cleanup_empty_dirs(EVMS_DIR_NAME);
 
-	status_message("Finished cleaning up the /dev/evms tree.\n");
+	status_message(_("Finished cleaning up the /dev/evms tree.\n"));
 
 	LOG_PROC_EXIT_VOID();
 	return;
@@ -2671,6 +2746,7 @@ int discover(list_anchor_t objects, boolean report_status) {
 	list_element_t iter1;
 	list_element_t iter2;
 	storage_object_t * obj;
+	list_anchor_t discover_objects = copy_list(objects);
 
 	boolean old_discover_in_progress;
 
@@ -2690,7 +2766,7 @@ int discover(list_anchor_t objects, boolean report_status) {
 	rc = discover_segments(&discover_list, report_status);
 
 	if (rc == 0) {
-		sort_list(&segments_list, compare_objects);
+		sort_list(&segments_list, compare_objects, NULL);
 
 		/* Add in regions for region discovery. */
 		LIST_FOR_EACH_SAFE(objects, iter1, iter2, obj) {
@@ -2703,12 +2779,12 @@ int discover(list_anchor_t objects, boolean report_status) {
 		rc = discover_regions(&discover_list, report_status);
 
 		if (rc == 0) {
-			sort_list(&regions_list, compare_objects);
+			sort_list(&regions_list, compare_objects, NULL);
 
 			/* Any objects left on the object list will be EVMS objects.
 			 * Put any remaining objects on the discover_list.
 			 */
-			merge_lists(&discover_list, objects, NULL);
+			merge_lists(&discover_list, objects, NULL, NULL);
 
 			rc = discover_evms_objects(&discover_list, report_status);
 			if (rc == 0) {
@@ -2726,9 +2802,9 @@ int discover(list_anchor_t objects, boolean report_status) {
 					rc = discover_associative_features(top_object_list, report_status);
 					if (rc == 0) {
 
-						sort_list(&EVMS_objects_list, compare_objects);
-						sort_list(&containers_list, compare_containers);
-						sort_list(&volumes_list, compare_volumes);
+						sort_list(&EVMS_objects_list, compare_objects, NULL);
+						sort_list(&containers_list, compare_containers, NULL);
+						sort_list(&volumes_list, compare_volumes, NULL);
 					}
 
 					destroy_list(top_object_list);
@@ -2739,6 +2815,9 @@ int discover(list_anchor_t objects, boolean report_status) {
 
 	discover_in_progress = old_discover_in_progress;
 
+	mark_for_activation(discover_objects);
+	destroy_list(discover_objects);
+
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
 }
@@ -2768,17 +2847,22 @@ int initial_discovery(void) {
 		discover_in_progress = TRUE;
 
 		rc = discover_logical_disks(&object_list, TRUE);
-		sort_list(&disks_list, compare_objects);
+		sort_list(&disks_list, compare_objects, NULL);
 
 		if (rc == 0) {
 			rc = discover(&object_list, TRUE);
+			if (replace_plugin != NULL) {
+				replace_plugin->functions.plugin->discover(NULL, NULL, TRUE);
+			}
 			cleanup_dev_evms_tree();
 		}
 
 		discover_in_progress = FALSE;
 
 	} else {
-		engine_user_message(NULL,NULL,"Error creating the %s directory: %s\n", EVMS_OBJECT_NODE_DIR, strerror(rc));
+		engine_user_message(NULL, NULL,
+				    _("Error creating the %s directory: %s\n"),
+				    EVMS_OBJECT_NODE_DIR, strerror(rc));
 	}
 
 	/*
@@ -2805,23 +2889,62 @@ int initial_discovery(void) {
 
 	LOG_DEFAULT("Discovery time: %02d:%02d:%02ld.%06ld\n", discover_hours, discover_minutes, discover_time.tv_sec, discover_time.tv_usec);
 
-	status_message("Discovery finished.\n");
+	status_message(_("Discovery finished.\n"));
 
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
 }
 
 
+static void mark_siblings_for_rediscover(storage_object_t * obj) {
+
+	list_element_t iter1;
+	list_element_t iter2;
+	storage_object_t * parent;
+	storage_object_t * sibling;
+
+	LOG_PROC_ENTRY();
+
+	LOG_DEBUG("Request to mark the siblings of %s for rediscover.\n", obj->name);
+
+	if (obj->consuming_container != NULL) {
+		LIST_FOR_EACH(obj->consuming_container->objects_consumed, iter2, sibling) {
+			LOG_DEBUG("Mark object %s for rediscover.\n", sibling->name);
+			sibling->flags |= SOFLAG_REDISCOVER;
+		}
+
+	} else if (!list_empty(obj->parent_objects)) {
+		LIST_FOR_EACH(obj->parent_objects, iter1, parent) {
+			LIST_FOR_EACH(parent->child_objects, iter2, sibling) {
+				LOG_DEBUG("Mark object %s for rediscover.\n", sibling->name);
+				sibling->flags |= SOFLAG_REDISCOVER;
+			}
+		}
+	}
+
+	LOG_PROC_EXIT_VOID();
+	return;
+}
+
+
 int evms_mark_for_rediscover(char * name) {
 
+	int i;
 	storage_object_t * obj;
 	storage_container_t * con;
 	logical_volume_t * vol;
 	list_element_t iter;
+	list_anchor_t object_lists[] = {&disks_list,
+					&segments_list,
+					&regions_list,
+					&EVMS_objects_list,
+					NULL};
 
 	LOG_PROC_ENTRY();
 
 	if (name == NULL) {
+		LOG_DEBUG("Request to mark everything for rediscover.\n");
+
 		/* Rediscover everything. Throw out everything but the disks. */
 		LIST_FOR_EACH(&disks_list, iter, obj) {
 			engine_discard(obj->parent_objects);
@@ -2836,50 +2959,32 @@ int evms_mark_for_rediscover(char * name) {
 		return 0;
 	}
 
+	LOG_DEBUG("Request to mark %s for rediscover.\n", name);
+
 	/* Must find the name by brute force search. */
-	LIST_FOR_EACH(&disks_list, iter, obj) {
-		if (strcmp(obj->name, name) == 0) {
-			engine_discard(obj->parent_objects);
-			obj->flags |= SOFLAG_REDISCOVER;
+	for (i = 0; object_lists[i] != NULL; i++) {
+		LIST_FOR_EACH(object_lists[i], iter, obj) {
+			if (strcmp(obj->name, name) == 0) {
+				mark_siblings_for_rediscover(obj);
 
-			LOG_PROC_EXIT_INT(0);
-			return 0;
-		}
-	}
+				engine_discard(obj->parent_objects);
+				if (obj->volume != NULL) {
+					discard_volume(obj->volume);
+					obj->volume = NULL;
+				}
+				LOG_DEBUG("Mark object %s for rediscover.\n", obj->name);
+				obj->flags |= SOFLAG_REDISCOVER;
 
-	LIST_FOR_EACH(&segments_list, iter, obj) {
-		if (strcmp(obj->name, name) == 0) {
-			engine_discard(obj->parent_objects);
-			obj->flags |= SOFLAG_REDISCOVER;
-
-			LOG_PROC_EXIT_INT(0);
-			return 0;
-		}
-	}
-
-	LIST_FOR_EACH(&regions_list, iter, obj) {
-		if (strcmp(obj->name, name) == 0) {
-			engine_discard(obj->parent_objects);
-			obj->flags |= SOFLAG_REDISCOVER;
-
-			LOG_PROC_EXIT_INT(0);
-			return 0;
-		}
-	}
-
-	LIST_FOR_EACH(&EVMS_objects_list, iter, obj) {
-		if (strcmp(obj->name, name) == 0) {
-			engine_discard(obj->parent_objects);
-			obj->flags |= SOFLAG_REDISCOVER;
-
-			LOG_PROC_EXIT_INT(0);
-			return 0;
+				LOG_PROC_EXIT_INT(0);
+				return 0;
+			}
 		}
 	}
 
 	LIST_FOR_EACH(&containers_list, iter, con) {
 		if (strcmp(con->name, name) == 0) {
 			engine_discard(con->objects_produced);
+			LOG_DEBUG("Mark container %s for rediscover.\n", con->name);
 			con->flags |= SCFLAG_REDISCOVER;
 
 			LOG_PROC_EXIT_INT(0);
@@ -2889,6 +2994,7 @@ int evms_mark_for_rediscover(char * name) {
 
 	LIST_FOR_EACH(&volumes_list, iter, vol) {
 		if (strcmp(vol->name, name) == 0) {
+			LOG_DEBUG("Mark volume %s for rediscover.\n", vol->name);
 			vol->flags |= VOLFLAG_REDISCOVER;
 
 			LOG_PROC_EXIT_INT(0);
@@ -2896,6 +3002,8 @@ int evms_mark_for_rediscover(char * name) {
 		}
 	}
 
+	LOG_DEBUG("%s not found.\n", name);
+
 	LOG_PROC_EXIT_INT(ENOENT);
 	return ENOENT;
 }
@@ -2904,7 +3012,6 @@ int evms_mark_for_rediscover(char * name) {
 int rediscover(void) {
 
 	int rc = 0;
-	STATIC_LIST_DECL(discard_list);
 	STATIC_LIST_DECL(plugin_discard_list);
 	STATIC_LIST_DECL(discover_list);
 	STATIC_LIST_DECL(object_list);
@@ -2917,10 +3024,6 @@ int rediscover(void) {
 
 	LIST_FOR_EACH(&disks_list, iter1, obj) {
 		if (obj->flags & SOFLAG_REDISCOVER) {
-			/*
-			 * Disks are not discarded for rediscover.  They are
-			 * simply put on the discover list.
-			 */
 			insert_thing(&discover_list, obj, INSERT_AFTER | EXCLUSIVE_INSERT, NULL);
 			obj->flags &= ~SOFLAG_REDISCOVER;
 		}
@@ -2928,51 +3031,22 @@ int rediscover(void) {
 
 	LIST_FOR_EACH(&segments_list, iter1, obj) {
 		if (obj->flags & SOFLAG_REDISCOVER) {
-			insert_thing(&discard_list, obj, INSERT_AFTER, NULL);
+			insert_thing(&discover_list, obj, INSERT_AFTER | EXCLUSIVE_INSERT, NULL);
+			obj->flags &= ~SOFLAG_REDISCOVER;
 		}
 	}
 	LIST_FOR_EACH(&regions_list, iter1, obj) {
 		if (obj->flags & SOFLAG_REDISCOVER) {
-			insert_thing(&discard_list, obj, INSERT_AFTER, NULL);
+			insert_thing(&discover_list, obj, INSERT_AFTER | EXCLUSIVE_INSERT, NULL);
+			obj->flags &= ~SOFLAG_REDISCOVER;
 		}
 	}
 	LIST_FOR_EACH(&EVMS_objects_list, iter1, obj) {
 		if (obj->flags & SOFLAG_REDISCOVER) {
-			insert_thing(&discard_list, obj, INSERT_AFTER, NULL);
+			insert_thing(&discover_list, obj, INSERT_AFTER | EXCLUSIVE_INSERT, NULL);
+			obj->flags &= ~SOFLAG_REDISCOVER;
 		}
 	}
-
-	while (!list_empty(&discard_list)) {
-		plugin_record_t * plugrec;
-
-		obj = (storage_object_t *) first_thing(&discard_list, NULL);
-		plugrec = obj->plugin;
-
-		/*
-		 * Put all objects that are managed by the same plug-in onto
-		 * separate list for the plug-ins discard();
-		 */
-		LIST_FOR_EACH_SAFE(&discard_list, iter1, iter2, obj) {
-			if (obj->plugin == plugrec) {
-				list_element_t iter;
-				storage_object_t * child;
-
-				remove_element(iter1);
-				insert_element(&plugin_discard_list, iter1, INSERT_AFTER, NULL);
-
-				/* Put the object's children on the discover list. */
-				LIST_FOR_EACH(obj->child_objects, iter, child) {
-					insert_thing(&discover_list, child, INSERT_AFTER | EXCLUSIVE_INSERT, NULL);
-				}
-			}
-		}
-
-		plugrec->functions.plugin->discard(&plugin_discard_list);
-
-		/* Deactivate the objects that were discarded. */
-		process_deactivate_list();
-	}
-
 	LIST_FOR_EACH_SAFE(&containers_list, iter1, iter2, con) {
 		if (con->flags & SCFLAG_REDISCOVER) {
 			list_element_t iter;
@@ -2987,9 +3061,6 @@ int rediscover(void) {
 		}
 	}
 
-	/*
-	 * The discover_list containes all the objects that are to be reexamined.
-	 */
 	if (!list_empty(&discover_list)) {
 		rc = discover(&discover_list, FALSE);
 	}
@@ -3006,27 +3077,31 @@ int evms_rediscover(void) {
 
 	LOG_PROC_ENTRY();
 
-	rc = rediscover();
-	if (rc != 0) {
-		LOG_WARNING("rediscover() returned error code %d: %s\n", rc, evms_strerror(rc));
-	}
-
-	/* Deactivate any volumes that were deleted. */
-	tmp_rc = delete_volumes();
+	/* Deactivate any volumes that were discarded. */
+	tmp_rc = deactivate_volumes_on_list(&volume_delete_list);
 	if (tmp_rc != 0) {
-		LOG_WARNING(" delete_volumes() returned error code %d: %s\n", tmp_rc, evms_strerror(tmp_rc));
+		LOG_WARNING("deactivate_volumes_on_list() returned error code %d: %s\n", tmp_rc, evms_strerror(tmp_rc));
 		if (rc == 0) {
 			rc = tmp_rc;
 		}
+	} else {
+		delete_volumes();
 	}
 
 	/* Deactivate any objects that were discarded. */
-	tmp_rc = process_deactivate_list();
+	tmp_rc = deactivate_objects_on_list(&deactivate_list);
 	if (tmp_rc != 0) {
-		LOG_WARNING("process_deactivate_list() returned error code %d: %s\n", tmp_rc, evms_strerror(tmp_rc));
+		LOG_WARNING("deactivate_objects_on_list() returned error code %d: %s\n", tmp_rc, evms_strerror(tmp_rc));
 		if (rc == 0) {
 			rc = tmp_rc;
 		}
+	} else {
+		delete_objects();
+	}
+
+	rc = rediscover();
+	if (rc != 0) {
+		LOG_WARNING("rediscover() returned error code %d: %s\n", rc, evms_strerror(rc));
 	}
 
 	tmp_rc = activate();
