@@ -26,6 +26,9 @@
 #include <errno.h>
 #include <mntent.h>
 #include <pthread.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 
 #include "fullengine.h"
 #include "engine.h"
@@ -345,6 +348,117 @@ int build_target_type_params(dm_target_t *target_list)
 	return rc;
 }
 
+/**
+ * load_dm_target_module
+ *
+ * Attempt to load the kernel module for the specified DM target.
+ **/
+static int load_dm_target_module(dm_target_type type)
+{
+	char *argv[3], buffer[64];
+	int status, rc = 0;
+	pid_t pid;
+
+	LOG_PROC_ENTRY();
+
+	snprintf(buffer, 64, "dm-%s", dm_target_type_info[type].name);
+
+	argv[0] = "modprobe";
+	argv[1] = buffer;
+	argv[2] = NULL;
+
+        pid = fork();
+
+	switch (pid) {
+	case -1:
+		rc = errno;
+		LOG_WARNING("fork() to run \"%s %s\" returned error %d: %s\n",
+			    argv[0], argv[1], rc, strerror(rc));
+		break;
+
+	case 0:
+		/* Child process */
+		execvp(argv[0], argv);
+
+		/* Should not get here unless execvp() fails. */
+		rc = errno;
+		LOG_WARNING("execvp() to run \"%s %s\" returned error %d: %s\n",
+			    argv[0], argv[1], rc, strerror(rc));
+
+		/* exit() kills the GUI.  Use _exit() instead. */
+		_exit(rc);
+		break;
+
+	default:
+		/* Parent process */
+		waitpid(pid, &status, 0);
+	}
+
+	if (!rc) {
+		if (WIFSIGNALED(status)) {
+			LOG_WARNING("\"%s %s\" was terminated by signal %s\n",
+				    argv[0], argv[1],
+				    sys_siglist[WTERMSIG(status)]);
+			rc = EINTR;
+		} else {
+			rc = WEXITSTATUS(status);
+			LOG_DEBUG("\"%s %s\" exited with error code %d: %s\n",
+				  argv[0], argv[1], rc, strerror(rc));
+		}
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * get_module_version
+ *
+ * Get the list of modules and versions from DM, find the desired target-type,
+ * and return the version of the currently loaded module.
+ **/
+static int get_module_version(dm_target_type type,
+			      evms_version_t *version)
+{
+	dm_module_list_t *module_list, *module;
+	int rc, module_loaded = FALSE;
+
+	LOG_PROC_ENTRY();
+
+start:
+	rc = dm_get_modules(&module_list);
+	if (rc) {
+		goto out;
+	}
+
+	for (module = module_list; module; module = module->next) {
+		rc = strncmp(dm_target_type_info[type].name,
+			     module->name, EVMS_NAME_SIZE);
+		if (!rc) {
+			break;
+		}
+	}
+
+	if (module) {
+		*version = module->version;
+		rc = 0;
+	} else if (!module_loaded) {
+		module_loaded = TRUE;
+		rc = load_dm_target_module(type);
+		if (!rc) {
+			goto start;
+		}
+	} else {
+		rc = ENOENT;
+	}
+
+	dm_deallocate_module_list(module_list);
+
+out:
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
 
 /**************************
  * Generic target routines.
@@ -928,20 +1042,48 @@ static int origin_translate_params(dm_target_t *target)
  * Multipath target routines.
  */
 
+/*
+ * There are now two versions of the parameter string for the multipath target.
+ * Version 1.0.3 and earlier of the module use a string of the form:
+ *
+ *   <num_groups> [ GROUP_SPEC ]+
+ *
+ *   GROUP_SPEC
+ *     <selector_name> <num_paths> <num_path_args> [ PATH_SPEC ]+
+ *
+ *     PATH_SPEC
+ *       <major>:<minor> [ <path_arg> ]*
+ *
+ * Versions of the module above 1.0.3 use a string of the form:
+ *
+ *   [ FEATURE_SPEC ] [ HW_HANDLER_SPEC ] [ GROUPING_SPEC ]
+ *
+ *   FEATURE_SPEC
+ *     <num_feature_args> [ <feature_arg> ]*
+ *
+ *   HW_HANDLER_SPEC
+ *     <num_hw_handler_args> [ <hw_handler_name> [ <hw_handler_arg> ]* ]*
+ *
+ *   GROUPING_SPEC
+ *     <num_groups> <initial_group> [ GROUP_SPEC ]+
+ *
+ *     GROUP_SPEC
+ *       [ SELECTOR_SPEC ] <num_paths> <num_per_path_args> [ PATH_SPEC ]+
+ *
+ *       SELECTOR_SPEC
+ *         <selector_name> <num_selector_args> [ <selector_arg> ]*
+ *
+ *       PATH_SPEC
+ *         <major>:<minor> [ <path_arg> ]*
+ */
+
 /**
  * multipath_build_params
  *
  * Generate an ASCII constructor string for a multipath target.
- * A multipath string has the form:
- *  <num_groups> [ GROUP_SPEC ]+
- *
- * GROUP_SPEC
- *  <selector-name> <num_paths> <num_path_args> [ PATH_SPEC ]+
- *
- * PATH_SPEC
- *  <major>:<minor> [ <path_arg> ]*
  **/
-static int multipath_build_params(dm_target_t *target)
+
+static int multipath_build_params_v1(dm_target_t *target)
 {
 	dm_target_multipath_t *mp = target->data.multipath;
 	char *format = (dm_get_version() == 3) ? " %x:%x" : " %u:%u";
@@ -988,20 +1130,85 @@ static int multipath_build_params(dm_target_t *target)
 	return rc;
 }
 
+static int multipath_build_params_v2(dm_target_t *target)
+{
+	dm_target_multipath_t *mp = target->data.multipath;
+	char *format = (dm_get_version() == 3) ? " %x:%x" : " %u:%u";
+	int i, j, sz = 0, maxlen, rc = ENOMEM;
+
+	LOG_PROC_ENTRY();
+
+	maxlen = 17 + mp->num_groups * 57;
+	for (i = 0; i < mp->num_groups; i++) {
+		maxlen += mp->group[i].num_paths *
+			  (22 + mp->group[i].num_path_args * 20);
+	}
+
+	target->params = get_string(maxlen);
+	if (target->params) {
+		rc = 0;
+
+		sz += snprintf(target->params + sz, maxlen - sz,
+			       "0 0 %u 1", mp->num_groups);
+
+		for (i = 0; i < mp->num_groups; i++) {
+			sz += snprintf(target->params + sz, maxlen - sz,
+				       " %s 0 %u %u",
+				       mp->group[i].selector,
+				       mp->group[i].num_paths,
+				       mp->group[i].num_path_args);
+
+			for (j = 0; j < mp->group[i].num_paths; j++) {
+				sz += snprintf(target->params + sz, maxlen - sz,
+					       format,
+					       mp->group[i].path[j].device.major,
+					       mp->group[i].path[j].device.minor);
+
+				if (mp->group[i].num_path_args) {
+					sz += snprintf(target->params + sz,
+						       maxlen - sz, " %s",
+						       mp->group[i].path[j].path_args);
+				}
+			}
+		}
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+static int multipath_build_params(dm_target_t *target)
+{
+	evms_version_t version;
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	rc = get_module_version(DM_TARGET_MULTIPATH, &version);
+	if (rc) {
+		goto out;
+	}
+
+	if (version.major == 1 &&
+	    version.minor == 0 &&
+	    version.patchlevel <= 3) {
+		rc = multipath_build_params_v1(target);
+	} else {
+		rc = multipath_build_params_v2(target);
+	}
+
+out:
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
 /**
  * multipath_translate_params
  *
- * Fill in a multipath target structure based on an ASCII table string of
- * the form:
- *  <num_groups> [ GROUP_SPEC ]+
- *
- * GROUP_SPEC
- *  <selector-name> <num_paths> <num_path_args> [ PATH_SPEC ]+
- *
- * PATH_SPEC
- *  <major>:<minor> [ <path_arg> ]*
+ * Fill in a multipath target structure based on the ASCII table string.
  **/
-static int multipath_translate_params(dm_target_t *target)
+
+static int multipath_translate_params_v1(dm_target_t *target)
 {
 	dm_target_multipath_t *mp = target->data.multipath;
 	char *params = target->params;
@@ -1067,14 +1274,148 @@ out:
 	return rc;
 }
 
+static int multipath_translate_params_v2(dm_target_t *target)
+{
+	dm_target_multipath_t *mp = target->data.multipath;
+	char *params = target->params;
+	u_int32_t num, i, j, k;
+	int sz, rc;
+
+	LOG_PROC_ENTRY();
+
+	/* We don't care about the "feature" arguments. */
+	rc = sscanf(params, "%u", &num);
+	if (rc != 1) {
+		rc = EINVAL;
+		goto out;
+	}
+	params = next_token(params);
+
+	for (i = 0; i < num; i++) {
+		params = next_token(params);
+	}
+
+	/* We don't care about the "hw-handler" arguments. */
+	rc = sscanf(params, "%u", &num);
+	if (rc != 1) {
+		rc = EINVAL;
+		goto out;
+	}
+	params = next_token(params);
+
+	if (num != 0) {
+		params = next_token(params);
+	}
+	for (i = 0; i < num; i++) {
+		params = next_token(params);
+	}
+
+	/* Number of priority-groups. */
+	rc = sscanf(params, "%u", &mp->num_groups);
+	if (rc != 1) {
+		rc = EINVAL;
+		goto out;
+	}
+
+	params = next_token(params);
+	/* We don't care about the initial-priority-group. */
+	params = next_token(params);
+
+	for (i = 0; i < mp->num_groups; i++) {
+		rc = sscanf(params, "%s %u", mp->group[i].selector, &num);
+		if (rc != 2) {
+			rc = EINVAL;
+			goto out;
+		}
+		params = next_token(params);
+		params = next_token(params);
+
+		/* We don't care about the selector arguments. */
+		for (j = 0; j < num; j++) {
+			params = next_token(params);
+		}
+
+		rc = sscanf(params, "%u %u", &mp->group[i].num_paths,
+			    &mp->group[i].num_path_args);
+		if (rc != 2) {
+			rc = EINVAL;
+			goto out;
+		}
+		params = next_token(params);
+		params = next_token(params);
+
+		mp->group[i].path = engine_alloc(mp->group[i].num_paths *
+						 sizeof(dm_path_t));
+		if (!mp->group[i].path) {
+			rc = ENOMEM;
+			goto out;
+		}
+
+		for (j = 0; j < mp->group[i].num_paths; j++) {
+			rc = translate_device(&params,
+					      &mp->group[i].path[j].device.major,
+					      &mp->group[i].path[j].device.minor);
+			if (rc) {
+				goto out;
+			}
+
+			for (k = 0, sz = 0; k < mp->group[i].num_path_args; k++) {
+				rc = sscanf(params, "%s",
+					    mp->group[i].path[j].path_args + sz);
+				if (rc != 1) {
+					rc = EINVAL;
+					goto out;
+				}
+
+				strncat(mp->group[i].path[j].path_args,
+					" ", DM_PATH_ARGS_SIZE);
+				sz = strlen(mp->group[i].path[j].path_args);
+				params = next_token(params);
+			}
+		}
+	}
+
+	rc = 0;
+
+out:
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+static int multipath_translate_params(dm_target_t *target)
+{
+	evms_version_t version;
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	rc = get_module_version(DM_TARGET_MULTIPATH, &version);
+	if (rc) {
+		goto out;
+	}
+
+	if (version.major == 1 &&
+	    version.minor == 0 &&
+	    version.patchlevel <= 3) {
+		rc = multipath_translate_params_v1(target);
+	} else {
+		rc = multipath_translate_params_v2(target);
+	}
+
+out:
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
 /**
  * multipath_pretranslate_params
  *
- * Parse an ASCII table string to determine how many multipath devices and
+ * Parse the ASCII table string to determine how many multipath devices and
  * groups are represented. Same format as used for multipath_translate_params().
  **/
-static int multipath_pretranslate_params(char *params, u_int32_t *num_devs,
-					 u_int32_t *num_groups)
+
+static int multipath_pretranslate_params_v1(char *params, u_int32_t *num_devs,
+					    u_int32_t *num_groups)
 {
 	int i, j, k, rc, num_paths, num_path_args;
 
@@ -1121,6 +1462,124 @@ out:
 	return rc;
 }
 
+static int multipath_pretranslate_params_v2(char *params, u_int32_t *num_devs,
+					    u_int32_t *num_groups)
+{
+	u_int32_t i, j, k, num, num_paths, num_path_args;
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	*num_devs = 0;
+
+	/* We don't care about the "feature" arguments. */
+	rc = sscanf(params, "%u", &num);
+	if (rc != 1) {
+		rc = EINVAL;
+		goto out;
+	}
+	params = next_token(params);
+
+	for (i = 0; i < num; i++) {
+		params = next_token(params);
+	}
+
+	/* We don't care about the "hw-handler" arguments. */
+	rc = sscanf(params, "%u", &num);
+	if (rc != 1) {
+		rc = EINVAL;
+		goto out;
+	}
+	params = next_token(params);
+
+	if (num != 0) {
+		params = next_token(params);
+	}
+	for (i = 0; i < num; i++) {
+		params = next_token(params);
+	}
+
+	/* Number of priority-groups. */
+	rc = sscanf(params, "%u", num_groups);
+	if (rc != 1) {
+		rc = EINVAL;
+		goto out;
+	}
+
+	params = next_token(params);
+	/* We don't care about the initial-priority-group. */
+	params = next_token(params);
+
+	for (i = 0; i < *num_groups; i++) {
+		num_paths = num_path_args = 0;
+
+		rc = sscanf(params, "%*s %u", &num);
+		if (rc != 1) {
+			rc = EINVAL;
+			goto out;
+		}
+		params = next_token(params);
+		params = next_token(params);
+
+		/* We don't care about the selector arguments. */
+		for (j = 0; j < num; j++) {
+			params = next_token(params);
+		}
+
+		rc = sscanf(params, "%u %u",
+			    &num_paths, &num_path_args);
+		if (rc != 2) {
+			rc = EINVAL;
+			goto out;
+		}
+		params = next_token(params);
+		params = next_token(params);
+
+		for (j = 0; j < num_paths; j++) {
+			params = next_token(params);
+			for (k = 0; k < num_path_args; k++) {
+				params = next_token(params);
+			}
+		}
+
+		*num_devs += num_paths;
+	}
+
+	rc = 0;
+
+out:
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+static int multipath_pretranslate_params(char *params, u_int32_t *num_devs,
+					 u_int32_t *num_groups)
+{
+	evms_version_t version;
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	rc = get_module_version(DM_TARGET_MULTIPATH, &version);
+	if (rc) {
+		goto out;
+	}
+
+	if (version.major == 1 &&
+	    version.minor == 0 &&
+	    version.patchlevel <= 3) {
+		rc = multipath_pretranslate_params_v1(params, num_devs,
+						      num_groups);
+	} else {
+		rc = multipath_pretranslate_params_v2(params, num_devs,
+						      num_groups);
+	}
+
+out:
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
 /**
  * multipath_allocate_target
  *
@@ -1390,6 +1849,277 @@ out:
 }
 
 
+/*************************
+ * RAID target routines.
+ */
+
+/**
+ * raid_build_params
+ *
+ * Generate an ASCII constructor string for a raid target.
+ * A raid string has the form:
+ *   <log_type> <num_custom_log_params> [<custom_log_params>]* 
+ *      <chunk_size> <shared_flag> <num_elements> 
+ *      [<logmajor>:<logminor> <logstart_lba>] array of log elements
+ *      [<major>:<minor> <start_lba>] array of data elements
+ *
+ * A NULL log_type will always be "core", num_custom_log_params is 0.
+ * chunk_size is in sectors
+ *
+ * This is only available using DM version 4.
+ **/
+static int raid_build_params(dm_target_t *target)
+{
+	dm_target_raid_t *raid = target->data.raid;
+	int max_len;
+	int i, sz, rc = ENOMEM;
+
+	LOG_PROC_ENTRY();
+
+	if (dm_get_version() == 3) {
+		LOG_PROC_EXIT_INT(EINVAL);
+		return EINVAL;
+	}
+
+	if (raid->log) {
+		sz = strlen(raid->log);
+	} else {
+		sz = strlen("core 0");
+	}
+        
+	max_len = sz + 20 + (raid->num_elements * 50);
+
+	target->params = get_string(max_len);
+	if (target->params) {
+		if (raid->log) {
+			strcpy(target->params, raid->log);
+		} else {
+			strcpy(target->params, "core 0");
+		}
+
+		sz += snprintf(target->params + sz, max_len - sz,
+			       " %u %u %u",
+			       raid->chunk_size, 
+			       raid->shared_flag,
+			       raid->num_elements);
+
+		for (i = 0; i < raid->num_elements; i++) {
+			/* If log == null, zero out the log devices. */
+			if (raid->log == NULL) {
+				raid->logdevices[i].major = 0;
+				raid->logdevices[i].minor = 0;
+				raid->logdevices[i].start = 0;
+			}
+			sz += snprintf(target->params + sz, max_len - sz,
+				       " %u:%u %"PRIu64,
+				       raid->logdevices[i].major,
+				       raid->logdevices[i].minor,
+				       raid->logdevices[i].start);
+		}
+
+		for (i = 0; i < raid->num_elements; i++) {
+			sz += snprintf(target->params + sz, max_len - sz,
+				       " %u:%u %"PRIu64,
+				       raid->devices[i].major,
+				       raid->devices[i].minor,
+				       raid->devices[i].start);
+		}
+		LOG_DEBUG("Target parameters: %s\n", target->params);
+		rc = 0;
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * raid_translate_params
+ *
+ * Fill in a raid target structure based on an ASCII table string 
+ * of the form shown above.
+ *
+ * This is only available using DM version 4.
+ **/
+static int raid_translate_params(dm_target_t *target)
+{
+	dm_target_raid_t *raid = target->data.raid;
+	char *params = target->params, *log;
+	int i, rc;
+
+	LOG_PROC_ENTRY();
+
+	if (dm_get_version() == 3) {
+		LOG_PROC_EXIT_INT(EINVAL);
+		return EINVAL;
+	}
+
+	/* Find the log string */
+	log = next_token(params);
+
+	rc = sscanf(log, "%u", &i);
+
+	/* Skip the count parameter */
+	log = next_token(log);
+
+	/* Skip all the log param */
+	while (i) {
+		log = next_token(log);
+		i--;
+	}
+
+	/* Set a 0 at the end of the log string */
+	log--;
+	*log = 0;
+	log++;
+
+	/* Allocate a buffer and copy the log data */
+	raid->log = engine_strdup(params);
+
+	params = log;
+
+	rc = sscanf(params, "%u %u %u", &raid->chunk_size,
+		    &raid->shared_flag, &raid->num_elements);
+	if (rc != 3) {
+		rc = EINVAL;
+		goto out;
+	}
+
+	params = next_token(params);
+	params = next_token(params);
+	params = next_token(params);
+
+	for (i = 0; i < raid->num_elements; i++) {
+		rc = translate_device(&params, &(raid->logdevices[i].major),
+				      &(raid->logdevices[i].minor));
+		if (rc) {
+			goto out;
+		}
+
+		rc = sscanf(params, "%"SCNu64, &(raid->logdevices[i].start));
+		if (rc != 1) {
+			rc = EINVAL;
+			goto out;
+		}
+
+		params = next_token(params);
+	}
+
+	for (i = 0; i < raid->num_elements; i++) {
+		rc = translate_device(&params, &(raid->devices[i].major),
+				      &(raid->devices[i].minor));
+		if (rc) {
+			goto out;
+		}
+
+		rc = sscanf(params, "%"SCNu64, &(raid->devices[i].start));
+		if (rc != 1) {
+			rc = EINVAL;
+			goto out;
+		}
+
+		params = next_token(params);
+	}
+
+	rc = 0;
+out:
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * raid_pretranslate_params
+ *
+ * Parse an ASCII table string to determine how many raid devices are
+ * represented. Same format as used for raid_translate_params().
+ **/
+static int raid_pretranslate_params(char *params, u_int32_t *num_devs,
+				    u_int32_t *num_groups)
+{
+	int i, rc;
+
+	LOG_PROC_ENTRY();
+
+	if (dm_get_version() != 4) {
+		LOG_PROC_EXIT_INT(EINVAL);
+		return EINVAL;
+	}
+
+	/* Skip log type at the start of the string. */
+	params = next_token(params);
+
+	rc = sscanf(params, "%u", &i);
+
+	/* Skip the count parameter */
+	params = next_token(params);
+
+	/* Skip all the log params. */
+	while (i) {
+		params = next_token(params);
+		i--;
+	}
+
+	/* Skip chunk size and shared flag. */
+	params = next_token(params);
+	params = next_token(params);
+
+	rc = sscanf(params, "%u", num_devs);
+	rc = (rc != 1) ? EINVAL : 0;
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * raid_allocate_target
+ *
+ * Allocate an array of log devices and devices to attach to the
+ * raid target structure.
+ **/
+static int raid_allocate_target(dm_target_t *target,
+				u_int32_t num_devs, u_int32_t num_groups)
+{
+	dm_target_raid_t *raid = target->data.raid;
+	int rc = 0;
+
+	LOG_PROC_ENTRY();
+
+	if (num_devs < 1) {
+		LOG_PROC_EXIT_INT(EINVAL);
+		return EINVAL;
+	}
+
+	raid->num_elements = num_devs;
+	raid->logdevices = engine_alloc(num_devs * sizeof(dm_device_t));
+	raid->devices = engine_alloc(num_devs * sizeof(dm_device_t));
+	if (!raid->devices || !raid->logdevices) {
+		engine_free(raid->logdevices);
+		engine_free(raid->devices);
+		rc = ENOMEM;
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * raid_deallocate_target
+ *
+ * Deallocate the array of logdevices and devices attached to the
+ * raid target structure.
+ **/
+static void raid_deallocate_target(dm_target_t *target)
+{
+	dm_target_raid_t *raid = target->data.raid;
+
+	LOG_PROC_ENTRY();
+
+	engine_free(raid->logdevices);
+	engine_free(raid->devices);
+	engine_free(raid->log);
+
+	LOG_PROC_EXIT_VOID();
+}
+
 /**
  * dm_target_type_info
  *
@@ -1487,5 +2217,29 @@ struct dm_target_type_info_t dm_target_type_info[] = {
 	  generic_allocate_target,
 	  generic_deallocate_target,
 	  0 },
+
+	{ "raid0",
+	  raid_build_params,
+	  raid_translate_params,
+	  raid_pretranslate_params,
+	  raid_allocate_target,
+	  raid_deallocate_target,
+	  sizeof(dm_target_raid_t) },
+
+	{ "raid1",
+	  raid_build_params,
+	  raid_translate_params,
+	  raid_pretranslate_params,
+	  raid_allocate_target,
+	  raid_deallocate_target,
+	  sizeof(dm_target_raid_t) },
+
+	{ "raid5",
+	  raid_build_params,
+	  raid_translate_params,
+	  raid_pretranslate_params,
+	  raid_allocate_target,
+	  raid_deallocate_target,
+	  sizeof(dm_target_raid_t) },
 };
 
