@@ -172,7 +172,8 @@ boolean list_empty(anchor_t * anchor) {
  */
 element_t * find_in_list(anchor_t         * anchor,
 			 void             * thing,
-			 compare_function_t compare) {
+			 compare_function_t compare,
+			 void             * user_data) {
 
 	element_t * element;
 
@@ -185,7 +186,7 @@ element_t * find_in_list(anchor_t         * anchor,
 
 	if (compare != NULL) {
 		INTERNAL_LIST_FOR_EACH(anchor, element) {
-			if (compare(thing, element->thing) == 0) {
+			if (compare(thing, element->thing, user_data) == 0) {
 				LOG_PROC_EXTRA_EXIT_PTR(element);
 				return element;
 			}
@@ -553,7 +554,7 @@ element_t * insert_element(anchor_t     * anchor,
 	}
 
 	if (flags & EXCLUSIVE_INSERT) {
-		element_t * el = find_in_list(anchor, element->thing, NULL);
+		element_t * el = find_in_list(anchor, element->thing, NULL, NULL);
 
 		if (el != NULL) {
 			/* Found the thing in the list.  Return its element. */
@@ -596,7 +597,7 @@ element_t * insert_thing(anchor_t     * anchor,
 	}
 
 	if (flags & EXCLUSIVE_INSERT) {
-		element_t * el = find_in_list(anchor, thing, NULL);
+		element_t * el = find_in_list(anchor, thing, NULL, NULL);
 
 		if (el != NULL) {
 			/* Found the thing in the list.  Return its element. */
@@ -889,7 +890,8 @@ int concatenate_lists(anchor_t * anchor1,
  */
 static int _merge_lists(anchor_t         * anchor1,
 			anchor_t         * anchor2,
-			compare_function_t compare) {
+			compare_function_t compare,
+			void             * user_data) {
 
 	anchor_t  * tmp_anchor;
 	element_t * element1;
@@ -955,7 +957,7 @@ static int _merge_lists(anchor_t         * anchor1,
 			 * Must compare the two things to see which
 			 * one goes first.
 			 */
-			if (compare(element1->thing, element2->thing) <= 0) {
+			if (compare(element1->thing, element2->thing, user_data) <= 0) {
 				insert_element = element1;
 				element1 = ELEMENT_PTR(element1->links.next);
 
@@ -981,7 +983,8 @@ static int _merge_lists(anchor_t         * anchor1,
 
 int merge_lists(anchor_t         * anchor1,
 		anchor_t         * anchor2,
-		compare_function_t compare) {
+		compare_function_t compare,
+		void             * user_data) {
 
 	LOG_PROC_EXTRA_ENTRY();
 
@@ -1003,7 +1006,7 @@ int merge_lists(anchor_t         * anchor1,
 		return 0;
 
 	} else {
-		int rc = _merge_lists(anchor1, anchor2, compare);
+		int rc = _merge_lists(anchor1, anchor2, compare, user_data);
 		LOG_PROC_EXTRA_EXIT_INT(rc);
 		return rc;
 	}
@@ -1016,7 +1019,8 @@ int merge_lists(anchor_t         * anchor1,
  * compare is not NULL.
  */
 static int _qsort_list(anchor_t         * anchor,
-		       compare_function_t compare) {
+		       compare_function_t compare,
+		       void             * user_data) {
 
 	int rc = 0;
 	anchor_t * tmp_anchor;
@@ -1063,17 +1067,17 @@ static int _qsort_list(anchor_t         * anchor,
 	set_list_owner(tmp_anchor);
 
 	/* Sort the two sub-lists. */
-	rc = _qsort_list(anchor, compare);
+	rc = _qsort_list(anchor, compare, user_data);
 
 	if (rc == 0) {
-		rc = _qsort_list(tmp_anchor, compare);
+		rc = _qsort_list(tmp_anchor, compare, user_data);
 	}
 
 	if (rc == 0) {
 		/*
 		 * Merge the lists back together using the comparison function provided.
 		 */
-		rc = _merge_lists(anchor, tmp_anchor, compare);
+		rc = _merge_lists(anchor, tmp_anchor, compare, user_data);
 	}
 
 	LOG_PROC_EXTRA_EXIT_INT(rc);
@@ -1085,7 +1089,8 @@ static int _qsort_list(anchor_t         * anchor,
  * Sort the elements in the list.
  */
 int sort_list(anchor_t         * anchor,
-	      compare_function_t compare) {
+	      compare_function_t compare,
+	      void             * user_data) {
 
 	int rc = 0;
 	anchor_t * copy;
@@ -1111,7 +1116,7 @@ int sort_list(anchor_t         * anchor,
 		return ENOMEM;
 	}
 
-	rc = _qsort_list(anchor, compare);
+	rc = _qsort_list(anchor, compare, user_data);
 
 	if (rc != 0) {
 		/*
