@@ -202,16 +202,25 @@ static void put_msg(uint cmd, size_t size, void * message) {
 
 static void worker_get_engine_api_version(void) {
 
+	size_t size;
 	evms_version_t engine_version;
-	flat_evms_version_t flat_version;
+	void * return_buf = NULL;
 
 	LOG_PROC_ENTRY();
 			
-	evms_get_api_version(&engine_version);
+	evms_sizeof_host_to_net(&size, evms_version_f, engine_version.major, engine_version.minor, engine_version.patchlevel);
 
-	evms_host_to_net(&flat_version, evms_version_f, engine_version.major, engine_version.minor, engine_version.patchlevel);
+	return_buf = calloc(1, size);
 
-	put_msg(EVMS_GET_API_VERSION | COMMAND_RESPONSE, sizeof(flat_version), &flat_version);
+	if (return_buf != NULL) {
+		evms_get_api_version(&engine_version);
+
+		evms_host_to_net(return_buf, evms_version_f, engine_version.major, engine_version.minor, engine_version.patchlevel);
+
+		put_msg(EVMS_GET_API_VERSION | COMMAND_RESPONSE, size, return_buf);
+
+		free(return_buf);
+	}
 	
 	LOG_PROC_EXIT_VOID();
 }
@@ -223,7 +232,7 @@ static int message_callback(char   * message_text,
 
 	int rc = 0;
 	size_t size;
-	flat_user_message_t * fum;
+	void * send_buf = NULL;
 	uint cmd;
 	void * reply;
 	int dummy;
@@ -232,15 +241,15 @@ static int message_callback(char   * message_text,
 			
 	rc = evms_sizeof_host_to_net(&size, user_message_args_f, message_text, answer, choices);
 	LOG_DEBUG("Flat user message is of size %u.\n", size);
-	fum = calloc(1, size);
+	send_buf = calloc(1, size);
 
-	if (fum != NULL) {
-		evms_host_to_net(fum, user_message_args_f,
+	if (send_buf != NULL) {
+		evms_host_to_net(send_buf, user_message_args_f,
 				 message_text,
 				 answer,
 				 choices);
 
-		put_msg(USER_MESSAGE, size, fum);
+		put_msg(USER_MESSAGE, size, send_buf);
 
 		do {
 			reply = get_msg(&cmd, &size, NULL);
@@ -268,7 +277,7 @@ static int message_callback(char   * message_text,
 			free(reply);
 		}
 
-		free(fum);
+		free(send_buf);
 
 	} else {
 		rc = ENOMEM;
@@ -283,7 +292,7 @@ static int progress_callback(progress_t * progress) {
 
 	int rc = 0;
 	size_t size;
-	flat_progress_t * fp;
+	void * send_buf;
 	uint cmd;
 	void * reply;
 
@@ -291,12 +300,12 @@ static int progress_callback(progress_t * progress) {
 			
 	evms_sizeof_host_to_net(&size, user_progress_args_f, progress);
 
-	fp = calloc(1, size);
+	send_buf = calloc(1, size);
 
-	if (fp != NULL) {
-		evms_host_to_net(fp, user_progress_args_f, progress);
+	if (send_buf != NULL) {
+		evms_host_to_net(send_buf, user_progress_args_f, progress);
 
-		put_msg(PROGRESS, size, fp);
+		put_msg(PROGRESS, size, send_buf);
 
 		do {
 			reply = get_msg(&cmd, &size, NULL);
@@ -318,7 +327,7 @@ static int progress_callback(progress_t * progress) {
 			free(reply);
 		}
 
-		free(fp);
+		free(send_buf);
 
 	} else {
 		rc = ENOMEM;
@@ -422,7 +431,7 @@ static void worker_get_plugin_list(void * msg) {
 
 	evms_plugin_code_t    type;
 	plugin_search_flags_t flags;
-	handle_array_t      * ha;
+	handle_array_t      * ha = NULL;
 	int rc;
 	size_t size;
 	void * return_buf = NULL;
@@ -684,7 +693,7 @@ static void worker_get_expand_points(void * msg) {
 	
 	int rc;
 	object_handle_t thing;
-	expand_handle_array_t * expand_points;
+	expand_handle_array_t * expand_points = NULL;
 	size_t size;
 	void * return_buf = NULL;
 
@@ -736,8 +745,8 @@ static void worker_get_expand_points(void * msg) {
 static void worker_expand(void * msg) {
 	
 	object_handle_t thing;
-	handle_array_t * objects;
-	option_array_t * options;
+	handle_array_t * objects = NULL;
+	option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -754,6 +763,9 @@ static void worker_expand(void * msg) {
 		
 	put_msg(EVMS_EXPAND | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(objects);
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -762,7 +774,7 @@ static void worker_get_shrink_points(void * msg) {
 	
 	int rc;
 	object_handle_t thing;
-	shrink_handle_array_t * shrink_points;
+	shrink_handle_array_t * shrink_points = NULL;
 	size_t size;
 	void * return_buf = NULL;
 
@@ -814,8 +826,8 @@ static void worker_get_shrink_points(void * msg) {
 static void worker_shrink(void * msg) {
 	
 	object_handle_t thing;
-	handle_array_t * objects;
-	option_array_t * options;
+	handle_array_t * objects = NULL;
+	option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -832,6 +844,9 @@ static void worker_shrink(void * msg) {
 		
 	put_msg(EVMS_SHRINK | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(objects);
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -998,7 +1013,7 @@ static void worker_get_extended_info(void * msg) {
 static void worker_set_info(void * msg) {
 
 	object_handle_t thing;
-	option_array_t * options;
+	option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1014,6 +1029,8 @@ static void worker_set_info(void * msg) {
 
 	put_msg(EVMS_SET_INFO | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1097,8 +1114,8 @@ static void worker_get_feature_list(void * msg) {
 static void worker_create(void * msg) {
 
 	plugin_handle_t  plugin;
-	handle_array_t * input_objects;
-	option_array_t * options;
+	handle_array_t * input_objects = NULL;
+	option_array_t * options = NULL;
 	handle_array_t * output_objects = NULL;
 	int rc;
 	size_t size;
@@ -1144,45 +1161,19 @@ static void worker_create(void * msg) {
 		put_msg(EVMS_CREATE | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 	}
 
+	evms_free(input_objects);
+	evms_free(options);
 	evms_free(output_objects);
 	
 	LOG_PROC_EXIT_VOID();
 }
 
 
-static void worker_transfer(void * msg) {
-	
-	object_handle_t  object;
-	plugin_handle_t  plugin;
-	object_handle_t  container;
-	option_array_t * options;
-	int rc;
-	u_int32_t net_rc;
-
-	LOG_PROC_ENTRY();
-			
-	evms_net_to_host(msg, evms_transfer_args_f,
-			 &object,
-			 &plugin,
-			 &container,
-			 &options);
-
-	rc = evms_transfer(object, plugin, container, options);
-
-	evms_host_to_net(&net_rc, evms_transfer_rets_f,
-			 rc);
-
-	put_msg(EVMS_TRANSFER | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
-
-	LOG_PROC_EXIT_VOID();
-}
-
-
 static void worker_assign(void * msg) {
 	
 	object_handle_t  object;
 	plugin_handle_t  plugin;
-	option_array_t * options;
+	option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1200,6 +1191,8 @@ static void worker_assign(void * msg) {
 
 	put_msg(EVMS_ASSIGN | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1268,7 +1261,7 @@ static void worker_get_object_list(void * msg) {
 static void worker_get_plugin_functions(void * msg) {
 
 	engine_handle_t         thing;
-	function_info_array_t * actions;
+	function_info_array_t * actions = NULL;
 	int rc;
 	size_t size;
 	void * return_buf = NULL;
@@ -1276,8 +1269,7 @@ static void worker_get_plugin_functions(void * msg) {
 	LOG_PROC_ENTRY();
 			
 	evms_net_to_host(msg, evms_get_plugin_functions_args_f,
-			 &thing,
-			 &actions);
+			 &thing);
 
 	rc = evms_get_plugin_functions(thing, &actions);
 
@@ -1323,8 +1315,8 @@ static void worker_do_plugin_function(void * msg) {
 
 	engine_handle_t  thing;
 	task_action_t    action;
-	handle_array_t * objects;
-	option_array_t * options;
+	handle_array_t * objects = NULL;
+	option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1343,6 +1335,9 @@ static void worker_do_plugin_function(void * msg) {
 
 	put_msg(EVMS_DO_PLUGIN_FUNCTION | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(objects);
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1423,7 +1418,7 @@ static void worker_add_feature_to_volume(void * msg) {
 
 	object_handle_t  volume;
 	plugin_handle_t  feature;
-        option_array_t * options;
+        option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1441,6 +1436,8 @@ static void worker_add_feature_to_volume(void * msg) {
 
 	put_msg(EVMS_ADD_FEATURE_TO_VOLUME | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1449,7 +1446,7 @@ static void worker_mkfs(void * msg) {
 
 	object_handle_t  volume;
 	plugin_handle_t  feature;
-        option_array_t * options;
+        option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1467,6 +1464,8 @@ static void worker_mkfs(void * msg) {
 
 	put_msg(EVMS_MKFS | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1474,7 +1473,7 @@ static void worker_mkfs(void * msg) {
 static void worker_fsck(void * msg) {
 
 	object_handle_t  volume;
-        option_array_t * options;
+        option_array_t * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1491,29 +1490,7 @@ static void worker_fsck(void * msg) {
 
 	put_msg(EVMS_FSCK | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
-	LOG_PROC_EXIT_VOID();
-}
-
-
-static void worker_defrag(void * msg) {
-
-	object_handle_t  volume;
-        option_array_t * options;
-	int rc;
-	u_int32_t net_rc;
-
-	LOG_PROC_ENTRY();
-			
-	evms_net_to_host(msg, evms_defrag_args_f,
-			 &volume,
-			 &options);
-
-	rc = evms_defrag(volume, options);
-
-	evms_host_to_net(&net_rc, evms_defrag_rets_f,
-			 rc);
-
-	put_msg(EVMS_DEFRAG | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
+	evms_free(options);
 
 	LOG_PROC_EXIT_VOID();
 }
@@ -1579,7 +1556,7 @@ static void worker_mount(void * msg) {
 
 	object_handle_t volume;
 	char          * mount_point;
-        char          * options;
+        char          * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1597,6 +1574,8 @@ static void worker_mount(void * msg) {
 
 	put_msg(EVMS_MOUNT | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1604,7 +1583,7 @@ static void worker_mount(void * msg) {
 static void worker_remount(void * msg) {
 
 	object_handle_t volume;
-        char          * options;
+        char          * options = NULL;
 	int rc;
 	u_int32_t net_rc;
 
@@ -1621,6 +1600,8 @@ static void worker_remount(void * msg) {
 
 	put_msg(EVMS_REMOUNT | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1628,9 +1609,9 @@ static void worker_remount(void * msg) {
 static void worker_create_container(void * msg) {
 
 	plugin_handle_t   plugin;
-	handle_array_t  * input_objects;
-	option_array_t  * options;
-	object_handle_t output_container;
+	handle_array_t  * input_objects = NULL;
+	option_array_t  * options = NULL;
+	object_handle_t   output_container;
 	int rc;
 	u_int32_t response_buf[2];
 
@@ -1649,6 +1630,9 @@ static void worker_create_container(void * msg) {
 
 	put_msg(EVMS_CREATE_CONTAINER | COMMAND_RESPONSE, sizeof(u_int32_t) * 2, response_buf);
 
+	evms_free(input_objects);
+	evms_free(options);
+
 	LOG_PROC_EXIT_VOID();
 }
 
@@ -1740,7 +1724,7 @@ static void worker_invoke_task(void * msg) {
 
 	int rc;
 	task_handle_t    task;
-	handle_array_t * resulting_objects;
+	handle_array_t * resulting_objects = NULL;
 	size_t size;
 	void * return_buf = NULL;
 
@@ -1845,7 +1829,7 @@ static void worker_get_acceptable_objects(void * msg) {
 
 	int rc;
 	task_handle_t    task;
-	handle_array_t * acceptable_object_list;
+	handle_array_t * acceptable_object_list = NULL;
 	size_t size;
 	void * return_buf = NULL;
 
@@ -1898,7 +1882,7 @@ static void worker_get_selected_objects(void * msg) {
 
 	int rc;
 	task_handle_t    task;
-	handle_array_t * selected_object_list;
+	handle_array_t * selected_object_list = NULL;
 	size_t size;
 	void * return_buf = NULL;
 
@@ -2005,8 +1989,8 @@ static void worker_set_selected_objects(void * msg) {
 
 	int rc;
 	task_handle_t             task;
-	handle_array_t          * selected_object_list;
-	declined_handle_array_t * declined_list;
+	handle_array_t          * selected_object_list = NULL;
+	declined_handle_array_t * declined_list = NULL;
 	task_effect_t             effect;
 	size_t size;
 	void * return_buf = NULL;
@@ -2053,6 +2037,7 @@ static void worker_set_selected_objects(void * msg) {
 		put_msg(EVMS_SET_SELECTED_OBJECTS | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
 	}
 
+	evms_free(selected_object_list);
 	evms_free(declined_list);
 
 	LOG_PROC_EXIT_VOID();
@@ -2115,7 +2100,7 @@ static void worker_get_option_descriptor(void * msg) {
 	int rc;
 	task_handle_t         task;
 	u_int32_t             option;
-	option_descriptor_t * descriptor;
+	option_descriptor_t * descriptor = NULL;
 	size_t size;
 	void * return_buf = NULL;
 
@@ -2180,6 +2165,8 @@ static void worker_set_option_value(void * msg) {
 
 	LOG_PROC_ENTRY();
 			
+	memset(&value, 0, sizeof(value_t));
+
 	evms_net_to_host(msg, evms_set_option_value_args_f,
 			 &task,
 			 &option,
@@ -2314,6 +2301,8 @@ static void worker_set_option_value_by_name(void * msg) {
 
 	LOG_PROC_ENTRY();
 			
+	memset(&value, 0, sizeof(value_t));
+
 	evms_net_to_host(msg, evms_set_option_value_by_name_args_f,
 			 &task,
 			 &option,
@@ -2512,10 +2501,6 @@ static void worker_one_handle_arg(uint cmd, void * msg) {
 			rc = evms_can_fsck(thing);
 			break;
 
-		case EVMS_CAN_DEFRAG:
-			rc = evms_can_defrag(thing);
-			break;
-
 		case EVMS_CAN_MOUNT:
 			rc = evms_can_mount(thing);
 			break;
@@ -2532,14 +2517,26 @@ static void worker_one_handle_arg(uint cmd, void * msg) {
 			rc = evms_can_remount(thing);
 			break;
 
-		case EVMS_CAN_REMOVE_FROM_CONTAINER:
-			rc = evms_can_remove_from_container(thing);
-                        break;
-
 		case EVMS_DESTROY_TASK:
 			rc = evms_destroy_task(thing);
 			break;
 
+		case EVMS_CAN_ACTIVATE:
+			rc = evms_can_activate(thing);
+			break;
+
+		case EVMS_ACTIVATE:
+			rc = evms_activate(thing);
+			break;
+
+		case EVMS_CAN_DEACTIVATE:
+			rc = evms_can_deactivate(thing);
+			break;
+
+		case EVMS_DEACTIVATE:
+			rc = evms_deactivate(thing);
+			break;
+
 		default:
 			LOG_SERIOUS("Got unexpected command %#x (%s).\n",
 				    cmd, msg_cmd_name(cmd));
@@ -2585,10 +2582,6 @@ static void worker_two_handle_arg(uint cmd, void * msg) {
 			rc = evms_can_mkfs(thing1, thing2);
 			break;
 
-		case EVMS_CAN_ADD_TO_CONTAINER:
-			rc = evms_can_add_to_container(thing1, thing2);
-			break;
-
 		default:
 			LOG_SERIOUS("Got unexpected command %#x (%s).\n",
 				    cmd, msg_cmd_name(cmd));
@@ -2604,6 +2597,149 @@ static void worker_two_handle_arg(uint cmd, void * msg) {
 }
 
 
+static void worker_metadata_backup(void * msg) {
+
+	char * dir;
+	int rc;
+	u_int32_t net_rc;
+
+	LOG_PROC_ENTRY();
+			
+	evms_net_to_host(msg, evms_metadata_backup_args_f,
+			 &dir);
+
+	rc = evms_metadata_backup(dir);
+
+	evms_host_to_net(&net_rc, evms_metadata_backup_rets_f,
+			 rc);
+
+	put_msg(EVMS_METADATA_BACKUP | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
+
+	LOG_PROC_EXIT_VOID();
+}
+
+
+static int worker_engine_read(void * msg) {
+
+	int rc = 0;
+	object_handle_t handle;
+	lsn_t           lsn;
+	sector_count_t  length;
+	u_int64_t       buff_size;
+	void          * buffer;
+	char rets_format[32];
+	size_t size;
+	void * return_buf = NULL;
+
+	LOG_PROC_ENTRY();
+
+	evms_net_to_host(msg, evms_engine_read_args_f,
+			 &handle,
+			 &lsn,
+			 &length);
+
+	buff_size = length << EVMS_VSECTOR_SIZE_SHIFT;
+	buffer = calloc(1, buff_size);
+	if (buffer == NULL) {
+		u_int32_t net_rc;
+		evms_host_to_net(&net_rc, int_f,
+				 ENOMEM);
+
+		put_msg(EVMS_ENGINE_READ | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
+	}
+
+	rc = evms_engine_read(handle, lsn, length, buffer);
+
+	if (rc == 0) {
+		sprintf(rets_format, "%sp{x%"PRIu64"}", evms_engine_read_rets_f, buff_size);
+
+		rc = evms_sizeof_host_to_net(&size, rets_format,
+					     rc,
+					     buffer);
+		if (rc == 0) {
+			return_buf = calloc(1, size);
+
+			if (return_buf == NULL) {
+				LOG_CRITICAL("Error getting memory for a return buffer.\n");
+				rc = ENOMEM;
+			}
+		}
+	}
+	
+	if (rc == 0) {
+		evms_host_to_net(return_buf, rets_format,
+				 rc,
+				 buffer);
+
+		put_msg(EVMS_ENGINE_READ | COMMAND_RESPONSE, size, return_buf);
+
+		free(return_buf);
+
+	} else {
+		u_int32_t net_rc;
+
+		evms_host_to_net(&net_rc, evms_engine_read_rets_f,
+				 rc);
+
+		put_msg(EVMS_ENGINE_READ | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
+	}
+
+	free(buffer);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+static int worker_engine_write(void * msg) {
+
+	int rc = 0;
+	char args_format[32];
+	object_handle_t handle;
+	lsn_t           lsn;
+	sector_count_t  length;
+	void          * buffer = NULL;
+	u_int32_t net_rc;
+
+	LOG_PROC_ENTRY();
+
+	/*
+	 * evms_engine_write_args_f does not include the "p{x<n>}" for the
+	 * fourth parameter which is the buffer of data to be written.
+	 * We must first get the length to know how big of a buffer to get.
+	 */
+	evms_net_to_host(msg, evms_engine_write_args_f,
+			 &handle,
+			 &lsn,
+			 &length);
+
+	/*
+	 * Now append the "p{x<n>}" to evms_engine_write_args_f and get all of the
+	 * arguments.
+	 */
+	sprintf(args_format, "%sp{x%"PRIu64"}",
+		evms_engine_write_args_f, length << EVMS_VSECTOR_SIZE_SHIFT);
+
+	evms_net_to_host(msg, args_format,
+			 &handle,
+			 &lsn,
+			 &length,
+			 &buffer);
+
+	rc = evms_engine_write(handle, lsn, length, buffer);
+
+	evms_host_to_net(&net_rc, int_f,
+			 rc);
+
+	put_msg(EVMS_ENGINE_WRITE | COMMAND_RESPONSE, sizeof(net_rc), &net_rc);
+
+	evms_free(buffer);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
 static void dispatch_commands(void) {
 
 	uint cmd;
@@ -2719,17 +2855,10 @@ static void dispatch_commands(void) {
 				worker_get_feature_list(msg);
 				break;
 
-//			case EVMS_DIRECT_PLUGIN_COMMUNICATION:
-//				break;
-//
 			case EVMS_CREATE:
 				worker_create(msg);
 				break;
 
-			case EVMS_TRANSFER:
-				worker_transfer(msg);
-				break;
-
 			case EVMS_ASSIGN:
 				worker_assign(msg);
 				break;
@@ -2799,7 +2928,6 @@ static void dispatch_commands(void) {
 
 			case EVMS_CAN_UNMKFS:
 			case EVMS_CAN_FSCK:
-			case EVMS_CAN_DEFRAG:
 				worker_one_handle_arg(cmd, msg);
 				break;
 
@@ -2815,10 +2943,6 @@ static void dispatch_commands(void) {
 				worker_fsck(msg);
 				break;
 
-			case EVMS_DEFRAG:
-				worker_defrag(msg);
-				break;
-
 			case EVMS_GET_VOLUME_LIST:
 				worker_get_volume_list(msg);
 				break;
@@ -2835,14 +2959,6 @@ static void dispatch_commands(void) {
 				worker_remount(msg);
 				break;
 
-			case EVMS_CAN_ADD_TO_CONTAINER:
-				worker_two_handle_arg(cmd, msg);
-				break;
-
-			case EVMS_CAN_REMOVE_FROM_CONTAINER:
-				worker_one_handle_arg(cmd, msg);
-				break;
-
 			case EVMS_CREATE_CONTAINER:
 				worker_create_container(msg);
 				break;
@@ -2911,6 +3027,25 @@ static void dispatch_commands(void) {
 				worker_rediscover(msg);
 				break;
 
+			case EVMS_CAN_ACTIVATE:
+			case EVMS_ACTIVATE:
+			case EVMS_CAN_DEACTIVATE:
+			case EVMS_DEACTIVATE:
+				worker_one_handle_arg(cmd, msg);
+				break;
+
+			case EVMS_METADATA_BACKUP:
+				worker_metadata_backup(msg);
+				break;
+
+			case EVMS_ENGINE_READ:
+				worker_engine_read(msg);
+				break;
+
+			case EVMS_ENGINE_WRITE:
+				worker_engine_write(msg);
+				break;
+
 			default:
 				if (cmd >= INVALID_COMMAND) {
 					LOG_WARNING("%d is not a valid command.\n", cmd);
