@@ -26,8 +26,8 @@
 #include <options.h>
 #include <appstructs.h>
 
-#define ENGINE_API_MAJOR_VERSION    9
-#define ENGINE_API_MINOR_VERSION    0
+#define ENGINE_API_MAJOR_VERSION    10
+#define ENGINE_API_MINOR_VERSION    1
 #define ENGINE_API_PATCH_LEVEL      0
 
 /*
@@ -124,7 +124,7 @@ int evms_get_plugin_list(evms_plugin_code_t    type,
 
 #define evms_get_plugin_list_rets_f \
 	int_f		            \
-	"p{" handle_array_f "}"
+	handle_array_f
 
 /*
  * Get the handle for a plug-in with a given ID.
@@ -173,7 +173,7 @@ int evms_changes_pending(boolean                 * result,
 #define evms_changes_pending_rets_f    \
 	int_f                          \
 	boolean_f	               \
-	"p{" change_record_array_f "}"
+	change_record_array_f
 
 /*
  * Write an entry to the Engine's log file.
@@ -210,6 +210,34 @@ int evms_commit_changes(void);
 #define evms_commit_changes_rets_f int_f
 
 
+/*
+ * Save a backup copy of the metadata that builds the current configuration.
+ * The metadata will be saved in a file named evms_metadata-yyymmddhhmmss in the
+ * "directory" given.  If "directory" is NULL, the default directory is
+ * /var/evms/metadata-backups/
+ */
+int evms_metadata_backup(const char * directory);
+
+#define evms_metadata_backup_args_f string_f
+
+#define evms_metadata_backup_rets_f int_f
+
+
+/*
+ * This function is the EVMS algorithm for calculating the CRC value for
+ * the data in the "buffer".
+ *
+ * The "initial_crc" parameter is the starting CRC.  If you are starting a new
+ * CRC calculation, then this should be set to 0xffffffff.  If you are
+ * continuing a CRC calculation (e.g.  all of the data did not fit in the buffer
+ * so you could not calculate the CRC in a single operation), then this is the
+ * CRC output by the last call to evms_calculate_crc.
+ */
+u_int32_t evms_calculate_crc(u_int32_t initial_crc,
+			     void    * buffer,
+			     u_int32_t buffer_size);
+
+
 /*
  * Close the EVMS Engine
  */
@@ -340,7 +368,7 @@ int evms_get_expand_points(object_handle_t           thing,
 
 #define evms_get_expand_points_rets_f \
 	int_f		              \
-	"p{" expand_handle_array_f "}"
+	expand_handle_array_f
 
 /*
  * evms_expand will determine the type of "thing" to be expanded (volume, evms
@@ -355,8 +383,8 @@ int evms_expand(object_handle_t  thing,
 
 #define evms_expand_args_f      \
 	object_handle_f         \
-	"p{" handle_array_f "}" \
-	"p{" option_array_f "}"
+	handle_array_f          \
+	option_array_f
 	
 #define evms_expand_rets_f int_f
 
@@ -377,7 +405,7 @@ int evms_get_shrink_points(object_handle_t           thing,
 
 #define evms_get_shrink_points_rets_f \
 	int_f		              \
-	"p{" shrink_handle_array_f "}"
+	shrink_handle_array_f
 
 /*
  * evms_shrink will determine the type of "thing" to be shrunk (volume, object,
@@ -392,8 +420,8 @@ int evms_shrink(object_handle_t  thing,
 
 #define evms_shrink_args_f      \
 	object_handle_f         \
-	"p{" handle_array_f "}" \
-	"p{" option_array_f "}"
+	handle_array_f          \
+	option_array_f
 	
 #define evms_shrink_rets_f int_f
 
@@ -437,7 +465,7 @@ int evms_get_info(object_handle_t          thing,
 
 #define evms_get_info_rets_f \
 	int_f		     \
-	"p{" handle_object_info_f "}"
+	handle_object_info_f
 
 /*
  * evms_get_extended_info will determine the type of "thing" (volume, object,
@@ -459,7 +487,7 @@ int evms_get_extended_info(object_handle_t           thing,
 	
 #define evms_get_extended_info_rets_f \
 	int_f 			      \
-	"p{" extended_info_array_f "}"
+	extended_info_array_f
 
 /*
  * evms_set_info will set information, specified in options, for a thing.
@@ -469,7 +497,7 @@ int evms_set_info(object_handle_t  object,
 
 #define evms_set_info_args_f    \
 	object_handle_f         \
-	"p{" option_array_f "}"
+	option_array_f
 
 #define evms_set_info_rets_f int_f
 
@@ -505,7 +533,7 @@ int evms_get_feature_list(object_handle_t    thing,
 
 #define evms_get_feature_list_rets_f \
 	int_f		             \
-	"p{" handle_array_f "}"
+	handle_array_f
 
 
 /***************************
@@ -524,30 +552,12 @@ int evms_create(plugin_handle_t    plugin,
 
 #define evms_create_args_f      \
 	plugin_handle_f	        \
-	"p{" handle_array_f "}" \
-	"p{" option_array_f "}"
+	handle_array_f          \
+	option_array_f
 	
 #define evms_create_rets_f \
 	int_f              \
-	"p{" handle_array_f "}"
-
-/*
- * evms_transfer will transfer an object to a particular plug-in and/or
- * container.  If container is 0, the plug-in will assign the object to its
- * default or unassigned container.
- */
-int evms_transfer(object_handle_t  object,
-		  plugin_handle_t  plugin,
-		  object_handle_t  container,
-		  option_array_t * options);
-
-#define evms_transfer_args_f    \
-	object_handle_f         \
-	plugin_handle_f         \
-	object_handle_f         \
-	"p{" option_array_f "}"
-	
-#define evms_transfer_rets_f int_f
+	handle_array_f
 
 /*
  * Assign a segment manager to a disk or segment using the specified options.
@@ -559,7 +569,7 @@ int evms_assign(object_handle_t  object,
 #define evms_assign_args_f      \
 	object_handle_f         \
 	plugin_handle_f         \
-	"p{" option_array_f "}"
+	option_array_f
 	
 #define evms_assign_rets_f int_f
 
@@ -608,7 +618,7 @@ int evms_get_object_list(object_type_t         object_type,
 	
 #define evms_get_object_list_rets_f \
 	int_f                       \
-	"p{" handle_array_f "}"
+	handle_array_f
 
 /*
  * Get an array of private actions that are supported by the plug-in that is
@@ -621,7 +631,7 @@ int evms_get_plugin_functions(engine_handle_t           thing,
 
 #define evms_get_plugin_functions_rets_f \
 	int_f			         \
-	"p{" function_info_array_f "}"
+	function_info_array_f
 
 /*
  * Execute a plug-in private action on a "thing".
@@ -634,8 +644,8 @@ int evms_do_plugin_function(engine_handle_t  thing,
 #define evms_do_plugin_function_args_f \
 	engine_handle_f                \
 	task_action_f                  \
-	"p{" handle_array_f "}"        \
-	"p{" option_array_f "}"
+	handle_array_f                 \
+	option_array_f
 	
 #define evms_do_plugin_function_rets_f int_f
 
@@ -766,7 +776,7 @@ int evms_add_feature_to_volume(object_handle_t  volume,
 #define evms_add_feature_to_volume_args_f \
 	object_handle_f                   \
 	plugin_handle_f                   \
-	"p{" option_array_f "}"
+	option_array_f
 
 #define evms_add_feature_to_volume_rets_f int_f
 
@@ -798,15 +808,6 @@ int evms_can_fsck(object_handle_t volume);
 
 #define evms_can_fsck_rets_f int_f
 
-/*
- * Can the volume be defragmented?
- */
-int evms_can_defrag(object_handle_t volume);
-
-#define evms_can_defrag_args_f object_handle_f
-
-#define evms_can_defrag_rets_f int_f
-
 /*
  * Install a file system on a volume.
  */
@@ -838,22 +839,10 @@ int evms_fsck(object_handle_t  volume,
 
 #define evms_fsck_args_f        \
 	object_handle_f         \
-	"p{" option_array_f "}"
+	option_array_f
 	
 #define evms_fsck_rets_f int_f
 
-/*
- * Defragment a volume.
- */
-int evms_defrag(object_handle_t  volume,
-		option_array_t * options);
-
-#define evms_defrag_args_f      \
-	object_handle_f         \
-	"p{" option_array_f "}"
-	
-#define evms_defrag_rets_f int_f
-
 /*
  * Get a list of handles for the volumes that are in the Engine's current
  * working set.  If the object handle of an FSIM is specified, the Engine will
@@ -876,7 +865,7 @@ int evms_get_volume_list(object_handle_t       fsim_handle,
 	
 #define evms_get_volume_list_rets_f \
 	int_f			    \
-	"p{" handle_array_f "}"
+	handle_array_f
 
 
 /*
@@ -947,27 +936,6 @@ int evms_remount(object_handle_t volume,
 * Storage Container functions *
 ******************************/
 
-/*
- * Can this object be added to the specified container?
- */
-int evms_can_add_to_container(object_handle_t object,
-			      object_handle_t container);
-
-#define evms_can_add_to_container_args_f \
-	object_handle_f                  \
-	object_handle_f
-	
-#define evms_can_add_to_container_rets_f int_f
-
-/*
- * Can this object be removed from the container that currently consumes it?
- */
-int evms_can_remove_from_container(object_handle_t object);
-
-#define evms_can_remove_from_container_args_f object_handle_f
-	
-#define evms_can_remove_from_container_rets_f int_f
-
 /*
  * Create a container for the given list of objects, using the specified
  * options.  A handle the new container is returned in output_container.
@@ -979,8 +947,8 @@ int evms_create_container(plugin_handle_t   plugin,
 
 #define evms_create_container_args_f \
 	plugin_handle_f	             \
-	"p{" handle_array_f "}"      \
-	"p{" option_array_f "}"
+	handle_array_f               \
+	option_array_f
 
 #define evms_create_container_rets_f \
 	int_f			     \
@@ -1009,7 +977,7 @@ int evms_get_container_list(plugin_handle_t          plugin_handle,
 	
 #define evms_get_container_list_rets_f \
 	int_f			       \
-	"p{" handle_array_f "}"
+	handle_array_f
 
 
 /*****************
@@ -1035,7 +1003,7 @@ int evms_invoke_task(task_handle_t      task,
 
 #define evms_invoke_task_rets_f \
 	int_f			\
-	"p{" handle_array_f "}"
+	handle_array_f
 
 int evms_destroy_task(task_handle_t task);
 
@@ -1063,7 +1031,7 @@ int evms_get_acceptable_objects(task_handle_t      task,
 
 #define evms_get_acceptable_objects_rets_f \
 	int_f				   \
-	"p{" handle_array_f "}"
+	handle_array_f
 
 int evms_get_selected_objects(task_handle_t      task,
 			      handle_array_t * * selected_object_list);
@@ -1072,7 +1040,7 @@ int evms_get_selected_objects(task_handle_t      task,
 
 #define evms_get_selected_objects_rets_f \
 	int_f			         \
-	"p{" handle_array_f "}"
+	handle_array_f
 
 int evms_get_selected_object_limits(task_handle_t task,
 				    u_int32_t   * minimum,
@@ -1092,11 +1060,11 @@ int evms_set_selected_objects(task_handle_t               task,
 
 #define evms_set_selected_objects_args_f \
 	task_handle_f			 \
-	"p{" handle_array_f "}"
+	handle_array_f
 	
 #define evms_set_selected_objects_rets_f \
 	int_f				 \
-	"p{" declined_handle_array_f "}" \
+	declined_handle_array_f          \
 	task_effect_f
 
 
@@ -1123,7 +1091,7 @@ int evms_get_option_descriptor(task_handle_t           task,
 
 #define evms_get_option_descriptor_rets_f \
 	int_f				  \
-	"p{" option_descriptor_f "}"
+	option_descriptor_f
 
 int evms_set_option_value(task_handle_t   task,
 			  u_int32_t       option,
@@ -1150,7 +1118,7 @@ int evms_get_option_descriptor_by_name(task_handle_t           task,
 	
 #define evms_get_option_descriptor_by_name_rets_f \
 	int_f					  \
-	"p{" option_descriptor_f "}"
+	option_descriptor_f
 
 int evms_set_option_value_by_name(task_handle_t   task,
 				  const char    * option_name,
@@ -1248,4 +1216,51 @@ int evms_get_config_string_array(char                 * key,
 				 int                  * count,
 				 const char * const * * array);
 
+
+/***********************
+* Activation functions *
+***********************/
+
+/*
+ * Can the volume or object be activated?
+ " "handle" is the handle for a volume or for an object.
+ */
+int evms_can_activate(object_handle_t handle);
+
+#define evms_can_activate_args_f object_handle_f
+
+#define evms_can_activate_rets_f int_f
+
+/*
+ * Activate the volume or object.
+ " "handle" is the handle for a volume or for an object.
+ * The volume or object will be activated when changes are committed.
+ */
+int evms_activate(object_handle_t handle);
+
+#define evms_activate_args_f object_handle_f
+
+#define evms_activate_rets_f int_f
+
+/*
+ * Can the volume or object be deactivated?
+ " "handle" is the handle for a volume or for an object.
+ */
+int evms_can_deactivate(object_handle_t handle);
+
+#define evms_can_deactivate_args_f object_handle_f
+
+#define evms_can_deactivate_rets_f int_f
+
+/*
+ * Dectivate the volume or object.
+ " "handle" is the handle for a volume or for an object.
+ * The volume or object will be deactivated when changes are committed.
+ */
+int evms_deactivate(object_handle_t handle);
+
+#define evms_deactivate_args_f object_handle_f
+
+#define evms_deactivate_rets_f int_f
+
 #endif
