@@ -53,7 +53,6 @@ plugin_record_t                *Seg_My_PluginRecord_Ptr=&segmgr_plugin_record;
 
 struct engine_functions_s      *EngFncs=NULL;
 
-
 /*
  * Called to revert back to original drive geometry that was
  * reported by the kernel. This is done when we have been
@@ -102,7 +101,7 @@ static int revert_drive_geometry( LOGICALDISK *ld )
 
 			// ask the user if it is Ok
 			{
-				char * choices[] = {"Yes", "No", NULL};
+				char * choices[] = {_("Yes"), _("No"), NULL};
 				int answer = 0;	    // Initialize to Yes ... revert to kernel reported geometry
 
 				QUESTION( &answer, choices,
@@ -324,7 +323,7 @@ static int SEG_CanDestroy( DISKSEG * seg )
  *
  */
 static int SEG_CanExpand( DISKSEG         *seg,		      // data segment
-			  sector_count_t  *expand_limit,      // ?
+			  sector_count_t  expand_limit,      // ?
 			  list_anchor_t          expansion_points ) // list to place expand object on
 {
 	DISKSEG              *freespace;
@@ -350,13 +349,14 @@ static int SEG_CanExpand( DISKSEG         *seg,		      // data segment
 		if ( freespace && ld ) {
 
 			// we can only expand in cylinder size amounts
-			if ( freespace->size >= get_cylinder_size(ld) ) {
+			if ( freespace->size >= get_cylinder_size(ld) &&
+			     expand_limit >= get_cylinder_size(ld)) {
 
 				expand_object = (expand_object_info_t *) EngFncs->engine_alloc( sizeof(expand_object_info_t) );
 				if (expand_object) {
 
 					expand_object->object          = seg;
-					expand_object->max_expand_size = freespace->size;
+					expand_object->max_expand_size = min(freespace->size, expand_limit);
 
 					if (EngFncs->insert_thing(expansion_points,expand_object,INSERT_BEFORE,NULL)) {
 						rc = 0;
@@ -475,7 +475,7 @@ static int SEG_CanExpandBy(storage_object_t * seg, sector_count_t *size)
  *  If not exact set new_size to closest higher value possible.
  */
 static int SEG_CanShrink( storage_object_t * seg,		// object to shrink
-			  sector_count_t   * shrink_limit,	// a delta size
+			  sector_count_t     shrink_limit,	// a delta size
 			  list_anchor_t            shrink_points )    // of type shrink_object_info_t,
 {
 	int                   rc = EINVAL;
@@ -503,7 +503,8 @@ static int SEG_CanShrink( storage_object_t * seg,		// object to shrink
 			SectorsPerCylinder = get_cylinder_size(ld);
 
 			// see if caller is trying to shrink smaller than min partition size
-			if ( seg->size > SectorsPerCylinder) {
+			if ( seg->size > SectorsPerCylinder &&
+			     shrink_limit >= SectorsPerCylinder) {
 
 				if (shrink_points) {
 
@@ -514,7 +515,7 @@ static int SEG_CanShrink( storage_object_t * seg,		// object to shrink
 						// minimum partition size is 1 cylinder ... then anything more than
 						// a cylinder is the max we can shrink.
 						shrink_object->object          = seg;
-						shrink_object->max_shrink_size = seg->size - SectorsPerCylinder;
+						shrink_object->max_shrink_size = min(seg->size - SectorsPerCylinder, shrink_limit);
 
 						if (EngFncs->insert_thing(shrink_points,shrink_object,INSERT_BEFORE,NULL)) {
 							rc = 0;
@@ -1012,7 +1013,6 @@ static int CreateDiskSegment( DISKSEG               * free_space,	    /* a free
 	 */
 	seg->size                 = seg_size;
 	seg->start                = seg_start_lba;
-	seg->flags               |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
 
 	if (Bootable) {
 		seg->flags           |= SOFLAG_BIOS_READABLE  ;
@@ -1561,6 +1561,7 @@ static int SEG_UnAssign( LOGICALDISK *ld )
 	int rc=EINVAL;
 	DISKSEG *seg;
 	list_element_t iter,iter2;
+	SEG_PRIVATE_DATA *pdata;
 
 
 	LOG_ENTRY();
@@ -1569,8 +1570,9 @@ static int SEG_UnAssign( LOGICALDISK *ld )
 	if ( i_can_modify_disk( ld ) == TRUE ) {
 
 		LIST_FOR_EACH_SAFE( ld->parent_objects, iter, iter2, seg ) {
+			pdata = seg->private_data;
 
-			if (seg->flags & SEG_IS_EBR) {
+			if (pdata->flags & SEG_IS_EBR) {
 
 				// blast EBR sector
 				KILL_SECTORS( ld, seg->start, 1);
@@ -1629,7 +1631,7 @@ static int SEG_DestroyEmbeddedSegment( LOGICALDISK *ld, DISK_PRIVATE_DATA *disk_
 
 		// ask the user if it is Ok
 		{
-			char * choices[] = {"Yes", "No", NULL};
+			char * choices[] = {_("Yes"), _("No"), NULL};
 			int answer = 0;	    // Initialize to Yes
 
 			QUESTION( &answer, choices,
@@ -1813,12 +1815,14 @@ static int SEG_DestroySegment( DISKSEG * seg, list_anchor_t child_objects )
 
 		rc = revert_drive_geometry(ld);
 		if (rc == 0) {
+			char number_buffer[64];
 
 			// tell the user
-			MESSAGE( "\nInfo, reverting to kernel reported geometry for drive %s.\n"
-				 "The new geometry is C= %"PRIu64" H= %d S= %d\n",
+			sprintf(number_buffer, "%"PRIu64, disk_pdata->geometry.cylinders);
+			MESSAGE( _("Information: reverting to kernel reported geometry for drive %s.\n"
+				   "The new geometry is C= %s H= %d S= %d\n"),
 				 ld->name,
-				 disk_pdata->geometry.cylinders,
+				 number_buffer,
 				 disk_pdata->geometry.heads,
 				 disk_pdata->geometry.sectors_per_track );
 		}
@@ -1950,6 +1954,15 @@ static int SEG_Expand( DISKSEG *seg, DISKSEG *expand_object, list_anchor_t  obje
 				// calc actual expand sector count
 				expand_sectors = end_lba - freespace->start + 1;
 
+				// ask the engine if this expand amount is ok.
+				rc = EngFncs->can_expand_by(seg, &expand_sectors);
+				if (rc) {
+					LOG_ERROR("Shrink of segment %s rejected by "
+						  "the engine.\n", seg->name);
+					LOG_EXIT_INT(rc);
+					return rc;
+				}
+
 				// expand the data segment
 				old_seg_size = seg->size;
 				seg->size   += expand_sectors;
@@ -1983,7 +1996,11 @@ static int SEG_Expand( DISKSEG *seg, DISKSEG *expand_object, list_anchor_t  obje
 	if (!rc) {
 		disk_pdata->flags |= DISK_HAS_CHANGES_PENDING;
 		if ( pdata->flags & SEG_IS_LOGICAL_PARTITION ) fixup_disk_extd_partition_dimensions( ld );
-		seg->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
+		seg->flags |= SOFLAG_DIRTY;
+		if (seg->flags & SOFLAG_ACTIVE) {
+			seg->flags |= SOFLAG_NEEDS_ACTIVATE;
+		}
+		
 	}
 
 
@@ -2075,6 +2092,15 @@ static int SEG_Shrink( storage_object_t * seg,
 				shrink_sector_count = (shrink_sector_count/SectorsPerCylinder)*SectorsPerCylinder;
 			}
 
+			// ask the engine if it's ok to shrink by this amount.
+			rc = EngFncs->can_shrink_by(seg, &shrink_sector_count);
+			if (rc) {
+				LOG_ERROR("Shrink of segment %s rejected by "
+					  "the engine.\n", seg->name);
+				LOG_EXIT_INT(rc);
+				return rc;
+			}
+
 			// resulting seg size
 			new_seg_size = seg->size - shrink_sector_count;
 
@@ -2115,7 +2141,10 @@ static int SEG_Shrink( storage_object_t * seg,
 				find_freespace_on_disk( ld );
 
 				// mark seg dirty for commit
-				seg->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
+				seg->flags |= SOFLAG_DIRTY;
+				if (seg->flags & SOFLAG_ACTIVE) {
+					seg->flags |= SOFLAG_NEEDS_ACTIVATE;
+				}
 
 			}
 
@@ -2224,7 +2253,7 @@ static int SEG_CommitChanges( storage_object_t *obj, uint phase )
 			     disk_pdata->flags & DISK_HAS_CHANGES_PENDING &&
 			     !(disk_pdata->flags & DISK_HAS_MOVE_PENDING) ) {
 
-				rc = Commit_Disk_Partition_Tables(ld);
+				rc = Commit_Disk_Partition_Tables(ld, obj, FALSE);
 
 				// if successful ... mark all segments on the disk clean
 				if (rc == 0) {
@@ -2259,6 +2288,22 @@ static int SEG_CommitChanges( storage_object_t *obj, uint phase )
 }
 
 
+static int SEG_BackupMetadata(DISKSEG *seg)
+{
+	LOGICALDISK *ld = get_logical_disk(seg);
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	if (i_can_modify_disk(ld) == TRUE &&
+	    seg->data_type != FREE_SPACE_TYPE) {
+		rc = Commit_Disk_Partition_Tables(ld, seg, TRUE);
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
 
 static int SEG_Read( DISKSEG        *seg,
 		     lsn_t           offset,
@@ -2398,10 +2443,10 @@ static int SEG_get_plugin_functions( storage_object_t        * object,
 
 			func_info->info[SEG_MOVE_OPTION_INDEX].function = EVMS_Task_Dos_Move_Segment;
 
-			SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].title, "Move" );
-			SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].verb,  "Move" );
-			SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].name, "Move" );
-			SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].help,  "Use this function to move a data segment." );
+			func_info->info[SEG_MOVE_OPTION_INDEX].title = EngFncs->engine_strdup( "Move" );
+			func_info->info[SEG_MOVE_OPTION_INDEX].verb = EngFncs->engine_strdup( _("Move") );
+			func_info->info[SEG_MOVE_OPTION_INDEX].name = EngFncs->engine_strdup( _("Move") );
+			func_info->info[SEG_MOVE_OPTION_INDEX].help = EngFncs->engine_strdup( _("Use this function to move a data segment.") );
 
 		}
 
@@ -2448,10 +2493,13 @@ static struct plugin_functions_s sft={
 	read:                                SEG_Read,
 	write:                               SEG_Write,
 	set_volume:                          SEG_set_volume,
+	can_activate:                        SEG_can_activate,
 	activate:                            SEG_activate,
 	deactivate:                          SEG_deactivate,
+	can_deactivate:                      SEG_can_deactivate,
 	get_plugin_functions:                SEG_get_plugin_functions,
 	plugin_function:                     SEG_plugin_function,
+	backup_metadata:                     SEG_BackupMetadata,
 
 	// the following routines found in segoptions.c
 	get_option_count:                    SEG_GetOptionCount,
@@ -2474,8 +2522,8 @@ static plugin_record_t segmgr_plugin_record = {
 
 	version:                            {MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL},
 
-	required_engine_api_version:        {13,0,0},
-	required_plugin_api_version:        {plugin: {12,0,0}},
+	required_engine_api_version:        {15,0,0},
+	required_plugin_api_version:        {plugin: {13,1,0}},
 
 	short_name:                         "DosSegMgr",
 	long_name:                          "DOS Segment Manager",
