@@ -32,7 +32,7 @@
 
 
 static list_anchor_t    dl_output_list;
-        
+
 
 /*
  *  Function: dl_read_metadata
@@ -151,7 +151,7 @@ static int dl_read_metadata( storage_object_t          * object,
                 rc = ENODATA;
         }
 
- 
+
         LOG_EXIT_INT(rc);
         return rc;
 }
@@ -163,7 +163,7 @@ static int dl_read_metadata( storage_object_t          * object,
  *  It is possible during discovery that we will find a missing child
  *  object already occupying the ordering table. When this happens, we
  *  simply need to remove the missing child and continue with discovery.
- *  This routine is called to test the ordering table for a missing 
+ *  This routine is called to test the ordering table for a missing
  *  child and to free it if one is found.
  */
 static int  dl_test_and_remove_missing_child( storage_object_t *drivelink,
@@ -178,7 +178,7 @@ static int  dl_test_and_remove_missing_child( storage_object_t *drivelink,
         pdata = (drivelink_private_data_t *) drivelink->private_data;
 
         index = dl_get_child_index_by_sn( drivelink, serial_number );
-        
+
         if ( index == BAD_DOT_INDEX ) {
                 LOG_ERROR("error, child serial number not found in link table\n");
                 LOG_EXIT_INT(rc);
@@ -205,7 +205,7 @@ static int  dl_test_and_remove_missing_child( storage_object_t *drivelink,
  *  allowing it to be added to the drivelink.
  */
 static int dl_verify_metadata( storage_object_t *drivelink,
-                               storage_object_t *child, 
+                               storage_object_t *child,
                                drivelink_metadata_t *md )
 {
         int rc=EINVAL, links=0, index=-1;
@@ -219,7 +219,7 @@ static int dl_verify_metadata( storage_object_t *drivelink,
 
         // find the object in the ordering table
         index = dl_get_child_index_by_sn( drivelink, md->child_serial_number );
-        
+
         /*
          *  TEST 1:  a childs serial number must actually occur in the ordering table
          */
@@ -228,9 +228,9 @@ static int dl_verify_metadata( storage_object_t *drivelink,
                 LOG_EXIT_INT(rc);
                 return rc;
         }
-    
-        /* 
-         * TEST 2: a child object may already have been found ... meaning there is an 
+
+        /*
+         * TEST 2: a child object may already have been found ... meaning there is an
          *         object with old metadata on it.  happens when old disk is added back
          *         to the system.
          */
@@ -282,6 +282,14 @@ static int dl_verify_metadata( storage_object_t *drivelink,
                 rc = EINVAL;
         }
 
+	/*
+	 * TEST 7:  Must have the same metadata sequence number
+	 */
+	else if ( md->sequence_number != pdata->sequence_number) {
+                LOG_ERROR("error, child metadata sequence number %"PRIu64" does not match the drivelink sequence number %"PRIu64".\n",
+			  md->sequence_number, pdata->sequence_number);
+                rc = EINVAL;
+	}
 
         rc=0;
         LOG_EXIT_INT(rc);
@@ -314,7 +322,7 @@ int dl_add_child_object_to_drivelink( storage_object_t       * drivelink,
         list_element_t iter1=NULL, iter2=NULL;
 
         LOG_ENTRY();
-        
+
         LOG_DEBUG("child= %s  child sn= 0x%X  size= %"PRIu64"\n", child->name, child_serial_number, child->size);
 
         // find the object in the ordering table
@@ -328,16 +336,16 @@ int dl_add_child_object_to_drivelink( storage_object_t       * drivelink,
         dot = &pdata->ordering_table[index];
 
         if (dot->child_vsize == 0) {
-        
+
                 // useable area = size - 2 feature header sectors - 2 copies of metadata
                 child_useable_size = child->size - evms_sectors;
-                
+
                 // useable area must an 8k (16 512 byte sectors) multiple, remaining sectors will
                 // be placed in the padding area.
                 padding = child_useable_size % 16;
 
                 child_useable_size -= padding;
-                
+
                 dot->child_vsize = child_useable_size;
         }
         else {
@@ -347,7 +355,7 @@ int dl_add_child_object_to_drivelink( storage_object_t       * drivelink,
                         padding = 0;
                 }
                 else {
-                        if (child->size >= (child_useable_size + evms_sectors)) {                       
+                        if (child->size >= (child_useable_size + evms_sectors)) {
                                 padding = child->size - child_useable_size - evms_sectors;
                         }
                         else {
@@ -380,22 +388,13 @@ int dl_add_child_object_to_drivelink( storage_object_t       * drivelink,
                 return rc;
         }
 
-        // We will only have the feature header parm if called during discovery.
-        // during discovery we'll keep the highest sequence number from all the
-        // child objects added to the link.
-        if (fh != NULL) {                               
-                if (fh->sequence_number > pdata->sequence_number) {
-                        pdata->sequence_number = fh->sequence_number;
-                }
-        }
-
         // fill in the information for this link in the drivelink linear map
         pdata->drive_link[index].padding         = padding;
         pdata->drive_link[index].sector_count    = child_useable_size;
         pdata->drive_link[index].serial_number   = child_serial_number;
         pdata->drive_link[index].index           = index;
         pdata->drive_link[index].object          = child;
- 
+
 	if ( dl_isa_missing_child(child) == TRUE ) {
 		pdata->drive_link[index].flags |= DL_FLAG_MISSING;
 	}
@@ -429,7 +428,7 @@ static int  dl_find_drivelink_object( char *name, storage_object_t **drivelink )
         list_anchor_t object_list = NULL;
         list_element_t iter;
         drivelink_private_data_t *pdata;
-        
+
         LOG_ENTRY();
 
         REQUIRE(name != NULL);
@@ -438,7 +437,7 @@ static int  dl_find_drivelink_object( char *name, storage_object_t **drivelink )
         *drivelink=NULL;
 
         rc = dl_get_drivelink_list(&object_list);
-        if (rc==0) {    
+        if (rc==0) {
                 LIST_FOR_EACH( object_list, iter, object ) {
                         pdata = (drivelink_private_data_t *) object->private_data;
                         if (strncmp(name,pdata->parent_object_name,EVMS_NAME_SIZE)==0){
@@ -450,7 +449,7 @@ static int  dl_find_drivelink_object( char *name, storage_object_t **drivelink )
         }
 
         if (*drivelink == NULL) rc=ENODEV;
-        
+
         LOG_EXIT_INT(rc);
         return rc;
 }
@@ -463,7 +462,7 @@ static int  dl_find_drivelink_object( char *name, storage_object_t **drivelink )
  *  data, registering names and serial numbers appropriate for a
  *  drivelink storage object.
  */
-static int dl_create_new_drivelink_object(char                  *drivelink_name, 
+static int dl_create_new_drivelink_object(char                  *drivelink_name,
                                           drivelink_metadata_t  *md,
                                           storage_object_t      *child,
                                           storage_object_t     **drivelink )
@@ -474,10 +473,10 @@ static int dl_create_new_drivelink_object(char                  *drivelink_name,
 
         LOG_ENTRY();
 
-        object = dl_malloc_drivelink_object();          
+        object = dl_malloc_drivelink_object();
         if (object) {
                 pdata = (drivelink_private_data_t *)object->private_data;
-                                
+
                 object->disk_group = child->disk_group;
 
                 // build cluster ready object name
@@ -489,32 +488,33 @@ static int dl_create_new_drivelink_object(char                  *drivelink_name,
 
                 // save drivelink name in pdata
                 strncpy(pdata->parent_object_name, drivelink_name, EVMS_NAME_SIZE);
-                                                                
-                pdata->parent_serial_number = md->parent_serial_number;
-                pdata->drive_link_count     = md->child_count;                          
 
-                memcpy( &pdata->ordering_table[0], 
-                        &md->ordering_table[0], 
+                pdata->parent_serial_number = md->parent_serial_number;
+		pdata->sequence_number      = md->sequence_number;
+                pdata->drive_link_count     = md->child_count;
+
+                memcpy( &pdata->ordering_table[0],
+                        &md->ordering_table[0],
                         sizeof(dot_entry_t)*md->child_count );
-                                
+
                 rc = EngFncs->register_name( object->name );
-                if (rc) {                                       
+                if (rc) {
                         LOG_ERROR("failed to register new parent drivelink storage object name\n");
                 }
 
                 // register drive link parent serial number
-                if (rc==0) {                            
+                if (rc==0) {
                         rc = dl_register_serial_number( md->parent_serial_number);
                         if (rc) {
                                 LOG_ERROR("failed to register new drivelink parent serial number\n");
                         }
                 }
-                
+
                 if (rc==0) {
                         *drivelink = object;
                 }
                 else {
-                        dl_free_drivelink_object(object);                       
+                        dl_free_drivelink_object(object);
                 }
         }
 
@@ -527,7 +527,7 @@ static int dl_create_new_drivelink_object(char                  *drivelink_name,
  *  Function:  dl_discover_child_objects
  *
  *  Called by the engine to discover drive link child objects. The objects
- *  in the list will be validated as drivelink objects and added to 
+ *  in the list will be validated as drivelink objects and added to
  *  drivelink storage objects.  If any object is not part of a drivelink it
  *  will simply be moved to the output list.
  */
@@ -541,8 +541,8 @@ static int dl_discover_child_objects( storage_object_t * obj )
         drivelink_private_data_t   *pdata;
         list_element_t              iter1, iter2;
 
-                                        
-        LOG_ENTRY();    
+
+        LOG_ENTRY();
 
         LOG_DEBUG("examining object %s\n", obj->name);
 
@@ -554,9 +554,9 @@ static int dl_discover_child_objects( storage_object_t * obj )
 
                 // find the drivelink object this child object belongs to ...
                 rc = dl_find_drivelink_object(obj->feature_header->object_name, &drivelink);
-                
-                if (rc) {  // drivelink storage object doesnt exist yet so we need to create it                   
-                        rc = dl_create_new_drivelink_object( obj->feature_header->object_name, 
+
+                if (rc) {  // drivelink storage object doesnt exist yet so we need to create it
+                        rc = dl_create_new_drivelink_object( obj->feature_header->object_name,
                                                              &md,
                                                              obj,
                                                              &drivelink );
@@ -565,28 +565,28 @@ static int dl_discover_child_objects( storage_object_t * obj )
                                 drivelink_object_created=TRUE;
                         }
                 }
-  
-                if (rc==0) { // validate the childs metadata by comparing it against the 
+
+                if (rc==0) { // validate the childs metadata by comparing it against the
                              // drivelink private data.
                         pdata = (drivelink_private_data_t *)drivelink->private_data;
 
-                        rc = dl_verify_metadata(drivelink, obj, &md);           
+                        rc = dl_verify_metadata(drivelink, obj, &md);
 
                         if (rc==0) { // if we suddenly found a child object for which we had a missing
                                      // child .... it means that somehow the engine didnt give us all
                                      // the child objects with which to construct the drivelink ... depth
-                                     // discovery problem in the engine ... no problem ... just replace 
+                                     // discovery problem in the engine ... no problem ... just replace
                                      // the missing child object with the link object we just got.
                                 rc = dl_test_and_remove_missing_child(drivelink, md.child_serial_number);
                         }
 
-                        if (rc==0) {  // metadata is confirmed so add the child object to 
+                        if (rc==0) {  // metadata is confirmed so add the child object to
                                       // the drivelink
-                                rc = dl_add_child_object_to_drivelink( drivelink, 
-                                                                       obj, 
+                                rc = dl_add_child_object_to_drivelink( drivelink,
+                                                                       obj,
                                                                        md.child_serial_number,
                                                                        obj->feature_header );
-                                if (rc==0) { // always clear the DISCOVERY COMPLETE flag when we 
+                                if (rc==0) { // always clear the DISCOVERY COMPLETE flag when we
                                              // succeed in adding a child object ... to force the
                                              // dl_discover() to rerun the mapping table work AND
                                              // refresh the device mapper kernel mapping. This prevents
@@ -596,7 +596,7 @@ static int dl_discover_child_objects( storage_object_t * obj )
                                 }
                         }
                 }
-        
+
                 if (rc) {
                         if (drivelink!=NULL) EngFncs->remove_thing(drivelink->child_objects, obj);
                         LIST_FOR_EACH_SAFE(obj->parent_objects, iter1, iter2, tmp_obj) {
@@ -645,11 +645,11 @@ int dl_discover( list_anchor_t input_list, list_anchor_t output_list, boolean fi
         drivelink_private_data_t *pdata;
 
         LOG_ENTRY();
- 
+
         REQUIRE(input_list != NULL && output_list != NULL);
 
         dl_output_list = output_list;
- 
+
         { // debug section
                 i=0;
                 count = EngFncs->list_count(input_list);
@@ -667,15 +667,15 @@ int dl_discover( list_anchor_t input_list, list_anchor_t output_list, boolean fi
         }
 
         // if this is the last time we will be called for discovery at this depth then ...
-        if (final_call == TRUE) {               
+        if (final_call == TRUE) {
                 rc = dl_get_drivelink_list(&object_list);
-                if (rc==0) { 
+                if (rc==0) {
                         LIST_FOR_EACH(object_list, iter, drivelink){
                                 LOG_DEBUG("drivelink: %s\n",drivelink->name);
                                 pdata = (drivelink_private_data_t *)drivelink->private_data;
                                 if ( !(pdata->flags & DL_DISCOVERY_COMPLETE) ) {
                                         LOG_DEBUG("...into final call processing for this drivelink.\n");
-                                        dl_table_fixup(drivelink);      // looks at the object link table and builds a 
+                                        dl_table_fixup(drivelink);      // looks at the object link table and builds a
                                                                         // missing child object for any NULL spot in
                                                                         // the ordered link table.
 
@@ -687,7 +687,7 @@ int dl_discover( list_anchor_t input_list, list_anchor_t output_list, boolean fi
                                                                         // that is appropriate for all the child objects.
 
                                         dl_get_devmap_info(drivelink);  // gets Device Mapper Info from kernel
-                                
+
                                         pdata->flags |= DL_DISCOVERY_COMPLETE;
                                 }
                                 else {
