@@ -24,7 +24,8 @@
 #ifndef COMMIT_HEADER
 #define COMMIT_HEADER 1
 
-int  commit_guid_partition_tables(LOGICALDISK *ld, uint phase);
+int  commit_guid_partition_tables(LOGICALDISK *ld, storage_object_t *parent,
+                                  uint phase, boolean backup);
 
 #endif
 
