@@ -23,8 +23,12 @@
 #ifndef _GPT_DEV_MAPPER_HEADER_
 #define _GPT_DEV_MAPPER_HEADER_ 1
 
+int     GPT_can_activate(storage_object_t *object );
+
 int     GPT_activate(storage_object_t *object );
 
+int     GPT_can_deactivate( storage_object_t *object );
+
 int     GPT_deactivate( storage_object_t *object );
 
 #endif
