@@ -104,18 +104,18 @@ static int snap_can_delete(storage_object_t * object)
  * snap_can_expand
  *
  * A snapshot volume can be expanded by allowing its child object to expand.
- * Origin volumes cannot expand at this time.
- *
- * FIXME: Are there any restrictions on when a snapshot can be expanded?
- *        Do we need to disallow it for disabled snapshots? For reset snapshots?
- *        For rollback snapshots? For inactive snapshots?
+ * Origin volumes cannot expand at this time. There aren't any restrictions
+ * on when you can expand a snapshot, because the snapshot device itself isn't
+ * expanded. Instead, the underlying device is expanded, and the snapshot
+ * device just happens to use the new space that's available for storing
+ * copy-on-writes.
  **/
 static int snap_can_expand(storage_object_t * object,
-			   sector_count_t * expand_limit,
+			   sector_count_t expand_limit,
 			   list_anchor_t expand_points)
 {
 	snapshot_volume_t * volume = object->private_data;
-	int rc = 0;
+	int rc = EINVAL;
 
 	LOG_ENTRY();
 
@@ -133,9 +133,7 @@ static int snap_can_expand(storage_object_t * object,
  * snap_can_expand_by
  *
  * If a snapshot's child object can expand, the snapshot object can expand by
- * the same amount.
- *
- * FIXME: Any restrictions from snap_can_expand() should also be enforced here.
+ * the same amount. Origin volumes cannot expand at this time.
  **/
 static int snap_can_expand_by(storage_object_t * object,
 			      sector_count_t * size)
@@ -156,11 +154,9 @@ static int snap_can_expand_by(storage_object_t * object,
 /**
  * snap_can_set_volume
  *
- * Origins:
- * - Cannot have their volume removed.
- * - May have their volume set. This will only happen if they are being renamed.
- * Snapshots:
- * - Cannot have their volume removed if they are waiting to be rolled-back.
+ * Origins cannot have their volume removed. Origins may have their volume
+ * set. This will only happen if they are being renamed. Snapshots no longer
+ * care whether they have a volume.
  **/
 static int snap_can_set_volume(storage_object_t * object,
 			       boolean creating)
@@ -170,15 +166,8 @@ static int snap_can_set_volume(storage_object_t * object,
 
 	LOG_ENTRY();
 
-	if (is_origin(volume)) {
-		if (!creating) {
-			rc = EINVAL;
-		}
-	} else {
-		if (!creating &&
-		    rollback_is_pending(volume)) {
-			rc = EINVAL;
-		}
+	if (is_origin(volume) && !creating) {
+		rc = EINVAL;
 	}
 
 	LOG_EXIT_INT(rc);
@@ -258,37 +247,21 @@ static int snap_discover(list_anchor_t input_objects,
 	/* Link the snapshot and the origin together. */
 	add_snapshot_to_origin(snap_volume, org_volume);
 
-	/* If the origin is not already active, it needs to be activated. */
-	if (!is_active(org_volume)) {
-		schedule_for_activate(org_volume);
-	}
-
-	/* If the snapshot is not already active, and the snapshot will become
-	 * a volume, and it isn't marked invalid in the metadata, it needs to
-	 * be activated.
-	 */
-	if (!is_active(snap_volume) &&
-	    !(snap_volume->child->feature_header->flags & EVMS_VOLUME_DATA_OBJECT)) {
-		check_snapshot_header(snap_volume);
-		if (!is_invalid(snap_volume)) {
-			schedule_for_activate(snap_volume);
-		}
-	}
-
 	if (is_invalid(snap_volume)) {
 		/* Display a warning if the snapshot is full/disabled. */
-		MESSAGE("Snapshot object %s discovered in full/disabled state. "
-			"Use the \"Reset\" function to reset the snapshot to "
-			"the current state of %s", snap_volume->parent->name,
+		MESSAGE(_("Snapshot object %s discovered in full/disabled state.  "
+			  "Use the \"Reset\" function to reset the snapshot to "
+			  "the current state of %s"),
+			snap_volume->parent->name,
 			org_volume->parent->volume->name);
 	} else if (rollback_is_pending(snap_volume)) {
 		/* Display a warning if the snapshot was interrupted in the
 		 * middle of a rollback.
 		 */
-		MESSAGE("Snapshot object %s needs to complete a roll-back "
-			"to %s. The operation was interrupted while in-"
-			"progress. Save changes to continue this roll-back "
-			"from the point it was interrupted.\n",
+		MESSAGE(_("Snapshot object %s needs to complete a roll-back "
+			  "to %s. The operation was interrupted while in-"
+			  "progress. Save changes to continue this roll-back "
+			  "from the point it was interrupted.\n"),
 			snap_volume->parent->name,
 			org_volume->parent->volume->name);
 		schedule_for_commit(snap_volume);
@@ -316,7 +289,7 @@ error:
  *			- volume to snapshot
  *			- chunk size
  *			- read-only or writeable
- * @new_objects:	List to output the new snapshot object to.
+ * @new_objects:	List to output the new snapshot and origin objects to.
  *
  * Create a new snapshot object.
  **/
@@ -330,7 +303,7 @@ static int snap_create(list_anchor_t objects,
 	snapshot_metadata_t * metadata = NULL;
 	char * org_vol_name, * snap_name;
 	u_int32_t chunk_size;
-	int writeable, rc = EINVAL;
+	int writeable, new_origin = FALSE, rc = EINVAL;
 
 	LOG_ENTRY();
 
@@ -399,21 +372,25 @@ static int snap_create(list_anchor_t objects,
 				  org_vol_name);
 			goto error;
 		}
+		new_origin = TRUE;
 	}
 
 	/* Link the snapshot and the origin together. */
 	add_snapshot_to_origin(snap_volume, org_volume);
 
-	/* If the origin is not already active, it needs to be activated. The
-	 * snapshot that was just created should not be activated until it is
-	 * made into a volume.
-	 */
-	if (!is_active(org_volume)) {
-		schedule_for_activate(org_volume);
-	}
-
+	/* This snapshot's metadata needs to be written during commit. */
 	schedule_for_commit(snap_volume);
 
+	/* Erase the snapshot header so it looks like a new snapshot to DM. */
+	KILL_SECTORS(snap_child, 0, 1);
+
+	/* Put newly created objects on the new_objects list. The engine will
+	 * use this list to decide which objects should be activated.
+	 */
+	if (new_origin) {
+		EngFncs->insert_thing(new_objects, org_volume->parent,
+				      INSERT_AFTER, NULL);
+	}
 	EngFncs->insert_thing(new_objects, snap_volume->parent,
 			      INSERT_AFTER, NULL);
 
@@ -434,14 +411,64 @@ error:
 /**
  * snap_discard
  *
- * Not sure what to do for this yet....
+ * Discard the specified snapshot or origin object. Unlike delete, the snapshot
+ * and origin objects are handled independently. However, the engine will
+ * ensure that all snapshots are discarded before their origin is discarded.
  **/
 static int snap_discard(list_anchor_t objects)
 {
+	storage_object_t * object, * child;
+	snapshot_volume_t * volume;
+	int rc = 0;
+
 	LOG_ENTRY();
 
-	LOG_EXIT_INT(0);
-	return 0;
+	/* The way the engine calls this routine, there should only be
+	 * one item on this list for the snapshot plugin.
+	 */
+	object = EngFncs->first_thing(objects, NULL);
+	if (!object) {
+		rc = EINVAL;
+		goto out;
+	}
+
+	volume = object->private_data;
+	child = volume->child;
+
+	if (is_origin(volume)) {
+		/* The origin must have no snapshots left. */
+		if (volume->count != 0) {
+			rc = EBUSY;
+			goto out;
+		}
+
+		unmake_parent_and_child(object, child);
+		child->volume->object = child;
+		EngFncs->free_evms_object(object);
+		if (is_active(volume)) {
+			schedule_for_delete(volume);
+		} else {
+			EngFncs->engine_free(volume);
+		}
+	} else {
+		unmake_parent_and_child(object, child);
+		remove_snapshot_from_origin(volume);
+		EngFncs->free_evms_object(object);
+
+		if (is_active(volume)) {
+			/* Wait until deactivation to delete all private data. */
+			schedule_for_delete(volume);
+		} else {
+			/* Delete all private data now. */
+			EngFncs->engine_free(volume->metadata);
+			EngFncs->engine_free(volume->sibling);
+			EngFncs->engine_free(volume);
+		}
+	}
+
+out:
+	LOG_EXIT_INT(rc);
+	return rc;
 }
 
 /**
@@ -484,7 +511,7 @@ static int snap_delete(storage_object_t * snap_parent,
 		/* Wait until deactivation to delete all private data. */
 		schedule_for_delete(snap_volume);
 	} else {
-		/* Deactivate all private data now. */
+		/* Delete all private data now. */
 		EngFncs->engine_free(snap_volume->metadata);
 		EngFncs->engine_free(snap_volume->sibling);
 		EngFncs->engine_free(snap_volume);
@@ -568,12 +595,11 @@ out:
 /**
  * snap_set_volume
  *
- * When a volume is added to a snapshot object, the first sector needs to be
- * zeroed so Device-Mapper will know it is a new snapshot.
- *
  * When a volume is added to an origin, it means the origin volume is being
  * renamed. The metadata for all of its snapshots must be updated to reflect
  * the new name.
+ *
+ * Snapshots no longer care whether they have EVMS volumes.
  **/
 static void snap_set_volume(storage_object_t * object,
 			    boolean creating)
@@ -585,25 +611,13 @@ static void snap_set_volume(storage_object_t * object,
 	LOG_DEBUG("%s volume on %s.\n",
 		  creating ? "Adding" : "Removing", object->name);
 
-	if (is_origin(volume)) {
-		if (creating) {
-			for (snap_volume = volume->next;
-			     snap_volume; snap_volume = snap_volume->next) {
-				strncpy(snap_volume->metadata->origin_volume,
-					skip_dev_path(object->volume->name),
-					EVMS_VOLUME_NAME_SIZE);
-				schedule_for_commit(snap_volume);
-			}
-		}
-	} else {
-		if (creating) {
-			KILL_SECTORS(volume->child, 0, 1);
-			schedule_for_commit(volume);
-			schedule_for_activate(volume);
-			mark_valid(volume);
-		} else {
-			activate_complete(volume);
-			schedule_for_deactivate(volume);
+	if (is_origin(volume) && creating) {
+		for (snap_volume = volume->next;
+		     snap_volume; snap_volume = snap_volume->next) {
+			strncpy(snap_volume->metadata->origin_volume,
+				skip_dev_path(object->volume->name),
+				EVMS_VOLUME_NAME_SIZE);
+			schedule_for_commit(snap_volume);
 		}
 	}
 
@@ -666,7 +680,7 @@ static int snap_commit_changes(storage_object_t * object,
 			}
 		}
 
-		rc = write_snapshot_metadata(volume);
+		rc = write_snapshot_metadata(volume, FALSE);
 		break;
 
 	case MOVE:
@@ -686,15 +700,30 @@ out:
 	return rc;
 }
 
+static int snap_can_activate(storage_object_t * object)
+{
+	LOG_ENTRY();
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
+static int snap_can_deactivate(storage_object_t * object)
+{
+	LOG_ENTRY();
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
 /**
  * snap_activate
  *
- * Activate a snapshot or an origin. Origins are now expected to be activated
+ * Activate a snapshot or an origin. Origins are expected to be activated
  * independently, and prior to activating any of their snapshots.
  **/
 static int snap_activate(storage_object_t * object)
 {
 	snapshot_volume_t * volume = object->private_data;
+	snapshot_volume_t * snap_volume;
 	int rc = 0;
 
 	LOG_ENTRY();
@@ -713,8 +742,30 @@ static int snap_activate(storage_object_t * object)
 	LOG_DEBUG("Activating %s.\n", object->name);
 
 	if (is_origin(volume)) {
+		/* Since we're activating the origin, we know none of the
+		 * snapshots can be active yet. We need to check that all the
+		 * snapshots will be activated. Any that are not scheduled for
+		 * activation need to be reset, since they will no longer be
+		 * valid.
+		 *
+		 * FIXME: Instead of always resetting the snapshot, we might
+		 * want to prompt the user whether to reset the snapshot, or
+		 * mark it for activation, or fail the activation of the origin.
+		 */
+		for (snap_volume = volume->next; snap_volume;
+		     snap_volume = snap_volume->next) {
+			if (!activate_is_pending(snap_volume)) {
+				rc = erase_snapshot_header(snap_volume, FALSE);
+				if (rc) {
+					goto out;
+				}
+			}
+		}
+
 		rc = activate_origin(volume);
+
 	} else {
+		/* Only activate the snapshot if its origin volume is active. */
 		if (is_active(volume->origin)) {
 			rc = activate_snapshot(volume);
 		} else {
@@ -764,6 +815,18 @@ static int snap_deactivate(storage_object_t * object)
 		deactivate_origin(volume);
 	} else {
 		deactivate_snapshot(volume);
+		/* If we deactivate the snapshot without deactivating the
+		 * origin, the snapshot must be reset.
+		 */
+		if (!deactivate_is_pending(volume->origin)) {
+			LOG_WARNING("Deactivating snapshot %s while origin %s "
+				    "remains active. The snapshot header must "
+				    "be erased to force a reset the next time "
+				    "this snapshot is activated.\n",
+				    volume->parent->name,
+				    volume->origin->parent->volume->name);
+			erase_snapshot_header(volume, FALSE);
+		}
 	}
 
 	deactivate_complete(volume);
@@ -941,24 +1004,20 @@ static int snap_get_info(storage_object_t * object,
 
 		for (i = 0, snap_volume = volume->next;
 		     snap_volume; snap_volume = snap_volume->next, i++) {
-			if (is_active(snap_volume)) {
-				SET_STRING(info->info[i].name, "SnapShot");
-				SET_STRING(info->info[i].title, "SnapShotted on");
-				SET_STRING(info->info[i].desc, "Snapshots of this volume");
-				info->info[i].type = EVMS_Type_String;
-				SET_STRING(info->info[i].value.s, snap_volume->parent->volume->name);
+			info->info[i].name = EngFncs->engine_strdup("SnapShot");
+			info->info[i].title = EngFncs->engine_strdup(_("SnapShotted on"));
+			info->info[i].desc = EngFncs->engine_strdup(_("Snapshots of this volume"));
+			info->info[i].type = EVMS_Type_String;
+			if (snap_volume->parent->volume) {
+				info->info[i].value.s = EngFncs->engine_strdup(snap_volume->parent->volume->name);
 			} else {
-				SET_STRING(info->info[i].name, "iSnapShot");
-				SET_STRING(info->info[i].title, "Inactive Snapshot on");
-				SET_STRING(info->info[i].desc, "This object is set up to snapshot this volume. If the object is made into a volume the snapshot will be reset and activated");
-				info->info[i].type = EVMS_Type_String;
-				SET_STRING(info->info[i].value.s, snap_volume->parent->name);
+				info->info[i].value.s = EngFncs->engine_strdup(snap_volume->parent->name);
 			}
 		}
 	} else {
 		/* Snapshot */
 		info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
-					     sizeof(extended_info_t) * 4);
+					     sizeof(extended_info_t) * 5);
 		if (!info) {
 			rc = ENOMEM;
 			goto out;
@@ -967,56 +1026,53 @@ static int snap_get_info(storage_object_t * object,
 		get_snapshot_state(volume);
 
 		if (is_active(volume)) {
-			SET_STRING(info->info[i].name, "Original");
-			SET_STRING(info->info[i].title, "SnapShot of");
-			SET_STRING(info->info[i].desc, "Indicates which volume this volume is a snapshot of.");
-			info->info[i].type = EVMS_Type_String;
-			SET_STRING(info->info[i].value.s, volume->origin->parent->volume->name);
+			info->info[i].name = EngFncs->engine_strdup("Original");
+			info->info[i].title = EngFncs->engine_strdup(_("SnapShot of"));
 		} else {
-			SET_STRING(info->info[i].name, "iOriginal");
-			SET_STRING(info->info[i].title, "Inactive SnapShot of");
-			SET_STRING(info->info[i].desc, "The volume which this object is a snapshot of.  Making this object a volume will reset and activate the snapshot.");
-			info->info[i].type = EVMS_Type_String;
-			SET_STRING(info->info[i].value.s, volume->origin->parent->volume->name);
+			info->info[i].name = EngFncs->engine_strdup("iOriginal");
+			info->info[i].title = EngFncs->engine_strdup(_("Inactive SnapShot of"));
 		}
+		info->info[i].desc = EngFncs->engine_strdup(_("Indicates which volume this volume is a snapshot of."));
+		info->info[i].type = EVMS_Type_String;
+		info->info[i].value.s = EngFncs->engine_strdup(volume->origin->parent->volume->name);
 		i++;
 
-		SET_STRING(info->info[i].name, "ChunkSize");
-		SET_STRING(info->info[i].title, "Chunk Size");
-		SET_STRING(info->info[i].desc, "The size of the chunks which are copied to this snapshot");
+		info->info[i].name = EngFncs->engine_strdup("ChunkSize");
+		info->info[i].title = EngFncs->engine_strdup(_("Chunk Size"));
+		info->info[i].desc = EngFncs->engine_strdup(_("The size of the chunks which are copied to this snapshot"));
 		info->info[i].type = EVMS_Type_Unsigned_Int32;
 		info->info[i].unit = EVMS_Unit_Sectors;
 		info->info[i].value.ui32 = volume->metadata->chunk_size;
 		i++;
 
-		SET_STRING(info->info[i].name, "Writeable");
-		SET_STRING(info->info[i].title, "Writeable");
-		SET_STRING(info->info[i].desc, "Whether snapshot is writeable or read-only");
+		info->info[i].name = EngFncs->engine_strdup("Writeable");
+		info->info[i].title = EngFncs->engine_strdup(_("Writeable"));
+		info->info[i].desc = EngFncs->engine_strdup(_("Whether snapshot is writeable or read-only"));
 		info->info[i].type = EVMS_Type_Boolean;
 		info->info[i].value.b = (volume->metadata->flags & SNAPSHOT_WRITEABLE);
 		i++;
 
-		SET_STRING(info->info[i].name, "State");
-		SET_STRING(info->info[i].title, "State");
-		SET_STRING(info->info[i].desc, "Current state of the snapshot.");
+		info->info[i].name = EngFncs->engine_strdup("State");
+		info->info[i].title = EngFncs->engine_strdup(_("State"));
+		info->info[i].desc = EngFncs->engine_strdup(_("Current state of the snapshot."));
 		info->info[i].type = EVMS_Type_String;
 		if (!is_active(volume)) {
-			SET_STRING(info->info[i].value.s, "Inactive");
+			info->info[i].value.s = EngFncs->engine_strdup(_("Inactive"));
 		} else if (is_invalid(volume)) {
-			SET_STRING(info->info[i].value.s, "Disabled/Full");
+			info->info[i].value.s = EngFncs->engine_strdup(_("Disabled/Full"));
 		} else if (rollback_is_pending(volume)) {
-			SET_STRING(info->info[i].value.s, "Pending Rollback");
+			info->info[i].value.s = EngFncs->engine_strdup(_("Pending Rollback"));
 		} else {
-			SET_STRING(info->info[i].value.s, "Active");
+			info->info[i].value.s = EngFncs->engine_strdup(_("Active"));
 		}
 		i++;
 
-		SET_STRING(info->info[i].name, "PercentFull");
-		SET_STRING(info->info[i].title, "Percent Full");
-		SET_STRING(info->info[i].desc, "Percentage of the snapshot device that has been used to save data from the origin.");
+		info->info[i].name = EngFncs->engine_strdup("PercentFull");
+		info->info[i].title = EngFncs->engine_strdup(_("Percent Full"));
+		info->info[i].desc = EngFncs->engine_strdup(_("Percentage of the snapshot device that has been used to save data from the origin."));
 		if (volume->percent_full < 0) {
 			info->info[i].type = EVMS_Type_String;
-			SET_STRING(info->info[i].value.s, "Unknown");
+			info->info[i].value.s = EngFncs->engine_strdup(_("Unknown"));
 		} else {
 			info->info[i].type = EVMS_Type_Unsigned_Int;
 			info->info[i].unit = EVMS_Unit_Percent;
@@ -1056,7 +1112,7 @@ static int snap_get_plugin_info(char * name,
 	}
 
 	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
-				     sizeof(extended_info_t)*5);
+				     sizeof(extended_info_t)*6);
 	if (!info) {
 		LOG_ERROR("Error allocating memory for plugin info array\n");
 		rc = ENOMEM;
@@ -1064,63 +1120,65 @@ static int snap_get_plugin_info(char * name,
 	}
 
 	/* Short Name */
-	SET_STRING(info->info[i].name, "ShortName");
-	SET_STRING(info->info[i].title, "Short Name");
-	SET_STRING(info->info[i].desc, "A short name given to this plugin");
+	info->info[i].name = EngFncs->engine_strdup("ShortName");
+	info->info[i].title = EngFncs->engine_strdup(_("Short Name"));
+	info->info[i].desc = EngFncs->engine_strdup(_("A short name given to this plug-in"));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, my_plugin_record->short_name);
+	info->info[i].value.s = EngFncs->engine_strdup(my_plugin_record->short_name);
 	i++;
 
 	/* Long Name */
-	SET_STRING(info->info[i].name, "LongName");
-	SET_STRING(info->info[i].title, "Long Name");
-	SET_STRING(info->info[i].desc, "A long name given to this plugin");
+	info->info[i].name = EngFncs->engine_strdup("LongName");
+	info->info[i].title = EngFncs->engine_strdup(_("Long Name"));
+	info->info[i].desc = EngFncs->engine_strdup(_("A longer, more descriptive name for this plug-in"));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, my_plugin_record->long_name);
+	info->info[i].value.s = EngFncs->engine_strdup(my_plugin_record->long_name);
 	i++;
 
 	/* Plugin Type */
-	SET_STRING(info->info[i].name, "Type");
-	SET_STRING(info->info[i].title, "Plugin Type");
-	SET_STRING(info->info[i].desc, "There are various types of plugins; each responsible for some kind of storage object.");
+	info->info[i].name = EngFncs->engine_strdup("Type");
+	info->info[i].title = EngFncs->engine_strdup(_("Plug-in Type"));
+	info->info[i].desc = EngFncs->engine_strdup(_("There are various types of plug-ins, each responsible for some kind of storage object or logical volume."));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, "Associative Feature");
+	info->info[i].value.s = EngFncs->engine_strdup(_("Associative Feature"));
 	i++;
 
 	/* Plugin Version */
-	SET_STRING(info->info[i].name, "Version");
-	SET_STRING(info->info[i].title, "Plugin Version");
-	SET_STRING(info->info[i].desc, "This is the version number of the plugin.");
+	info->info[i].name = EngFncs->engine_strdup("Version");
+	info->info[i].title = EngFncs->engine_strdup(_("Plug-in Version"));
+	info->info[i].desc = EngFncs->engine_strdup(_("This is the version number of the plug-in."));
 	info->info[i].type = EVMS_Type_String;
 	snprintf(buffer, 50, "%d.%d.%d",
 		 my_plugin_record->version.major,
 		 my_plugin_record->version.minor,
 		 my_plugin_record->version.patchlevel);
-	SET_STRING(info->info[i].value.s, buffer);
+	info->info[i].value.s = EngFncs->engine_strdup(buffer);
 	i++;
 
 	/* Required Engine Services Version */
-	SET_STRING(info->info[i].name, "Required_Engine_Version");
-	SET_STRING(info->info[i].title, "Required Engine Services Version");
-	SET_STRING(info->info[i].desc, "This is the version of the Engine services that this plug-in requires.  It will not run on older versions of the Engine services.");
+	info->info[i].name = EngFncs->engine_strdup("Required_Engine_Version");
+	info->info[i].title = EngFncs->engine_strdup(_("Required Engine Services Version"));
+	info->info[i].desc = EngFncs->engine_strdup(_("This is the version of the Engine services that this plug-in requires.  "
+						      "It will not run on older versions of the Engine services."));
 	info->info[i].type = EVMS_Type_String;
 	snprintf(buffer, 50, "%d.%d.%d",
 		 my_plugin_record->required_engine_api_version.major,
 		 my_plugin_record->required_engine_api_version.minor,
 		 my_plugin_record->required_engine_api_version.patchlevel);
-	SET_STRING(info->info[i].value.s, buffer);
+	info->info[i].value.s = EngFncs->engine_strdup(buffer);
 	i++;
 
 	/* Required Plug-in API Version */
-	SET_STRING(info->info[i].name, "Required_Plugin_Version");
-	SET_STRING(info->info[i].title, "Required Plug-in API Version");
-	SET_STRING(info->info[i].desc, "This is the version of the Engine plug-in API that this plug-in requires.  It will not run on older versions of the Engine plug-in API.");
+	info->info[i].name = EngFncs->engine_strdup("Required_Plugin_Version");
+	info->info[i].title = EngFncs->engine_strdup(_("Required Plug-in API Version"));
+	info->info[i].desc = EngFncs->engine_strdup(_("This is the version of the Engine plug-in API that this plug-in requires.  "
+						      "It will not run on older versions of the Engine plug-in API."));
 	info->info[i].type = EVMS_Type_String;
 	snprintf(buffer, 50, "%d.%d.%d",
 		 my_plugin_record->required_plugin_api_version.plugin.major,
 		 my_plugin_record->required_plugin_api_version.plugin.minor,
 		 my_plugin_record->required_plugin_api_version.plugin.patchlevel);
-	SET_STRING(info->info[i].value.s, buffer);
+	info->info[i].value.s = EngFncs->engine_strdup(buffer);
 	i++;
 
 	info->count = i;
@@ -1180,21 +1238,20 @@ static int snap_write(storage_object_t * object,
  * can_reset
  *
  * To reset the snapshot to the current state of the origin, the snapshot must
- * - have an EVMS volume (the only way it will be active without an EVMS volume
- *   is if the volume has just been deleted).
+ * - be active.
  * - not be pending a rollback.
  * - not be mounted (can't change the state of the snapshot out from under the
  *   filesystem).
  **/
 static int can_reset(snapshot_volume_t * snap_volume)
 {
-	char * mount_point;
-	int rc;
+	logical_volume_t * lvol;
+	int rc = 0;
 
 	LOG_ENTRY();
 
-	if (!snap_volume->parent->volume) {
-		LOG_DETAILS("Snapshot %s is already pending deactivation.\n",
+	if (!is_active(snap_volume)) {
+		LOG_DETAILS("Snapshot %s is not active. Nothing to reset.\n",
 			    snap_volume->parent->name);
 		rc = EINVAL;
 		goto out;
@@ -1207,13 +1264,10 @@ static int can_reset(snapshot_volume_t * snap_volume)
 		goto out;
 	}
 
-	rc = EngFncs->is_mounted(snap_volume->parent->volume->dev_node,
-				 &mount_point);
-	if (rc) {
-		LOG_DETAILS("Snapshot %s is mounted on %s. Please unmount "
-			    "before resetting.\n",
-			    snap_volume->parent->volume->dev_node, mount_point);
-		EngFncs->engine_free(mount_point);
+	if (!EngFncs->is_offline(snap_volume->parent, &lvol)) {
+		LOG_DETAILS("Snapshot %s is mounted. Please unmount before "
+			    "resetting.\n", snap_volume->parent->name);
+		rc = EINVAL;
 		goto out;
 	}
 
@@ -1253,7 +1307,7 @@ static int snap_get_plugin_functions(storage_object_t * object,
 	/* Allocate the function info array. */
 	fia = EngFncs->engine_alloc(sizeof(function_info_array_t) +
 				    sizeof(function_info_t) *
-				    (SNAPSHOT_FUNCTION_COUNT - 1));
+				    SNAPSHOT_FUNCTION_COUNT);
 	if (!fia) {
 		LOG_CRITICAL("Error allocating memory for function info array "
 			     "for %s.\n", object->name);
@@ -1266,11 +1320,12 @@ static int snap_get_plugin_functions(storage_object_t * object,
 
 	/* Can the snapshot be reset? */
 	fia->info[0].function = SNAPSHOT_FUNCTION_RESET;
-	SET_STRING(fia->info[0].name, "reset");
-	SET_STRING(fia->info[0].title, "Reset");
-	SET_STRING(fia->info[0].verb, "Reset");
-	SET_STRING(fia->info[0].help, "Reset the snapshot to the current "
-				      "state of the origin volume.");
+	fia->info[0].name = EngFncs->engine_strdup("reset");
+	fia->info[0].title = EngFncs->engine_strdup(_("Reset"));
+	fia->info[0].verb = EngFncs->engine_strdup(_("Reset"));
+	fia->info[0].help = EngFncs->engine_strdup(_("Reset the snapshot to the "
+						     "current state of the "
+						     "origin volume."));
 
 	rc = can_reset(volume);
 	if (rc) {
@@ -1279,11 +1334,12 @@ static int snap_get_plugin_functions(storage_object_t * object,
 
 	/* Can the snapshot be rolled-back? */
 	fia->info[1].function = SNAPSHOT_FUNCTION_ROLLBACK;
-	SET_STRING(fia->info[1].name, "rollback");
-	SET_STRING(fia->info[1].title, "Roll Back");
-	SET_STRING(fia->info[1].verb, "Roll Back");
-	SET_STRING(fia->info[1].help, "Roll back the contents of the snapshot "
-				      "to the original volume.");
+	fia->info[1].name = EngFncs->engine_strdup("rollback");
+	fia->info[1].title = EngFncs->engine_strdup(_("Roll Back"));
+	fia->info[1].verb = EngFncs->engine_strdup(_("Roll Back"));
+	fia->info[1].help = EngFncs->engine_strdup(_("Roll back the contents of "
+                                                     "the snapshot to the "
+						     "original volume."));
 
 	rc = can_rollback(volume, FALSE);
 	if (rc) {
@@ -1306,8 +1362,8 @@ static int snap_plugin_function(storage_object_t * object,
 				option_array_t * options)
 {
 	snapshot_volume_t * volume = object->private_data;
-	char * reset_choices[3] = { "Reset", "Don't Reset", NULL };
-	char * rollback_choices[3] = { "Rollback", "Don't Rollback", NULL };
+	char * reset_choices[3] = { _("Reset"), _("Don't Reset"), NULL };
+	char * rollback_choices[3] = { _("Rollback"), _("Don't Rollback"), NULL };
 	int answer = 0, rc = EINVAL;
 
 	LOG_ENTRY();
@@ -1327,26 +1383,29 @@ static int snap_plugin_function(storage_object_t * object,
 			break;
 		}
 
-		EngFncs->user_message(my_plugin_record, &answer, reset_choices,
-				      "WARNING: Selecting \"%s\" will cause "
-				      "all data saved in the snapshot \"%s\" "
-				      "to be lost, and the snapshot will be "
-				      "reset to the current state of the "
-				      "origin volume \"%s\".\n\n"
-				      "Do not mount the snapshot volume until "
-				      "after saving changes.",
-				      reset_choices[0], object->volume->name,
-				      volume->origin->parent->volume->name);
+		QUESTION(&answer, reset_choices,
+			 _("WARNING: Selecting \"Reset\" will cause "
+			   "all data saved in the snapshot \"%s\" "
+			   "to be lost, and the snapshot will be "
+			   "reset to the current state of the "
+			   "origin volume \"%s\".\n\n"
+			   "Do not mount the snapshot volume until "
+			   "after saving changes."),
+			 object->name,
+			 volume->origin->parent->volume->name);
 		if (answer != 0) {
 			rc = E_CANCELED;
 			break;
 		}
 
-		/* Reset is equivalent to removing and recreating
-		 * the EVMS volume.
+		/* Erase the snapshot header, mark the snapshot valid and dirty,
+		 * and schedule for reactivation.
 		 */
-		snap_set_volume(object, FALSE);
-		snap_set_volume(object, TRUE);
+		KILL_SECTORS(volume->child, 0, 1);
+		schedule_for_commit(volume);
+		mark_valid(volume);
+		schedule_for_activate(volume);
+		schedule_for_deactivate(volume);
 
 		break;
 
@@ -1358,17 +1417,16 @@ static int snap_plugin_function(storage_object_t * object,
 			break;
 		}
 
-		EngFncs->user_message(my_plugin_record, &answer,
-				      rollback_choices,
-				      "WARNING: Selecting \"%s\" will cause "
-				      "all data saved in the snapshot \"%s\" "
-				      "to be copied back to the origin volume "
-				      "\"%s\". The current state of the origin "
-				      "volume will be lost. Both the snapshot "
-				      "and the origin volumes must remain un-"
-				      "mounted until the rollback is complete.\n",
-				      rollback_choices[0], object->volume->name,
-				      volume->origin->parent->volume->name);
+		QUESTION(&answer, rollback_choices,
+			 _("WARNING: Selecting \"Rollback\" will cause "
+			   "all data saved in the snapshot \"%s\" "
+			   "to be copied back to the origin volume "
+			   "\"%s\". The current state of the origin "
+			   "volume will be lost. Both the snapshot "
+			   "and the origin volumes must remain un-"
+			   "mounted until the rollback is complete.\n"),
+			 object->volume->name,
+			 volume->origin->parent->volume->name);
 		if (answer != 0) {
 			rc = E_CANCELED;
 			break;
@@ -1391,6 +1449,44 @@ out:
 	return rc;
 }
 
+/**
+ * snap_backup_metadata
+ *
+ * Save the snapshot metadata to the engine's metadata-backup database. We
+ * only save data for the snapshot, since there's no metadata on the origin
+ * side. We also save a zeroed header, so if the snapshot volume is restored,
+ * it will appear as a new, valid snapshot.
+ **/
+static int snap_backup_metadata(storage_object_t *object)
+{
+	snapshot_volume_t * volume = object->private_data;
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	if (is_origin(volume)) {
+		goto out;
+	}
+
+	rc = write_snapshot_metadata(volume, TRUE);
+	if (rc) {
+		goto out;
+	}
+
+	rc = erase_snapshot_header(volume, TRUE);
+	if (rc) {
+		goto out;
+	}
+
+	rc = EngFncs->save_metadata(volume->parent->name,
+				    volume->origin->parent->volume->name,
+				    0, 0, NULL);
+
+out:
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
 /* Snapshot function table and plugin record. */
 static plugin_functions_t snapshot_functions = {
 	.setup_evms_plugin		= snap_setup_evms_plugin,
@@ -1409,6 +1505,8 @@ static plugin_functions_t snapshot_functions = {
 	.commit_changes			= snap_commit_changes,
 	.activate			= snap_activate,
 	.deactivate			= snap_deactivate,
+	.can_activate			= snap_can_activate,
+	.can_deactivate			= snap_can_deactivate,
 	.get_option_count		= snap_get_option_count,
 	.init_task			= snap_init_task,
 	.set_option			= snap_set_option,
@@ -1418,7 +1516,8 @@ static plugin_functions_t snapshot_functions = {
 	.read				= snap_read,
 	.write				= snap_write,
 	.get_plugin_functions		= snap_get_plugin_functions,
-	.plugin_function		= snap_plugin_function
+	.plugin_function		= snap_plugin_function,
+	.backup_metadata		= snap_backup_metadata,
 };
 
 plugin_record_t Snapshot_Plugin = {
@@ -1431,14 +1530,14 @@ plugin_record_t Snapshot_Plugin = {
 		.patchlevel	= PATCH_LEVEL
 	},
 	.required_engine_api_version = {
-		.major		= 13,
+		.major		= 15,
 		.minor		= 0,
 		.patchlevel	= 0
 	},
 	.required_plugin_api_version = {
 		.plugin = {
-			.major		= 12,
-			.minor		= 0,
+			.major		= 13,
+			.minor		= 1,
 			.patchlevel	= 0
 		}
 	},
