@@ -4,27 +4,26 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  * Module: container_cb.h
  */
 
-gboolean can_expand_container(object_handle_t container);
-
-gboolean can_shrink_container(object_handle_t container);
-
 void on_expand_container_clist_realize(GtkWidget * widget, gpointer user_data);
 
 void on_shrink_container_clist_realize(GtkWidget * widget, gpointer user_data);
 
 void on_container_plugin_clist_realize(GtkWidget * widget, gpointer user_data);
+
+void on_resize_container_button_clicked(GtkButton * button, gpointer user_data);
+
