@@ -4,16 +4,16 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  * Module: volume_cb.c
@@ -42,10 +42,10 @@
  *
  *   Description:
  *      This routine initiates the evms_create_compatibility_volume() API call
- *      using the handle of the object associated with the last row 
+ *      using the handle of the object associated with the last row
  *      selected. It then creates and displays the results popup which
  *      destroys both windows when dismissed.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - address of user data bound with signal (not used)
@@ -65,12 +65,12 @@ void on_create_compat_volume_button_clicked(GtkButton * button, gpointer user_da
 /*
  *
  *   void on_create_compat_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of storage objects that can be used to create a compatibility
  *      volume.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - address of user data bound with signal (not used)
@@ -90,10 +90,10 @@ void on_create_compat_volume_clist_realize(GtkWidget * widget, gpointer user_dat
  *
  *   Description:
  *      This routine initiates the evms_create_volume API call
- *      using the handle of the object associated with the last row 
+ *      using the handle of the object associated with the last row
  *      selected. It then creates and displays the results popup which
  *      destroys both windows when dismissed.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - address of user data bound with signal (not used)
@@ -113,12 +113,12 @@ void on_create_evms_volume_button_clicked(GtkButton * button, gpointer user_data
 /*
  *
  *   void on_create_evms_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of storage objects that can be used to create an evms
  *      volume.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - address of user data bound with signal (not used)
@@ -135,17 +135,17 @@ void on_create_evms_volume_clist_realize(GtkWidget * widget, gpointer user_data)
 /*
  *
  *   GtkWidget *create_create_evms_volume_window (GtkSignalFunc, gpointer)
- *   
+ *
  *   Description:
  *      This routine creates the dialog that allows creating an EVMS volume
  *      from the objects in the selection list. It is essentially a standard
  *      selection dialog modified slightly to allow specifying the name of
  *      the volume.
- * 
+ *
  *   Entry:
  *      clist_realize_sighandler - function that will populate list when realized
  *      user_data                - address of user data to bind with signal handlers
- *      
+ *
  *   Exit:
  *      Returns the id of the dialog window created.
  *
@@ -170,7 +170,7 @@ GtkWidget *create_create_evms_volume_window(GtkSignalFunc clist_realize_sighandl
 
 		/*
 		 * Before we show the window, setup the signal handlers
-		 * that will populate the clist when the clist is realized. 
+		 * that will populate the clist when the clist is realized.
 		 * Also initialize the "Cancel" button to dismiss the window.
 		 * Setup the the "Next" button to invoke the task and display
 		 * a results window. The "Next" button's sensitivity is reliant
@@ -284,7 +284,7 @@ GtkWidget *create_create_evms_volume_window(GtkSignalFunc clist_realize_sighandl
  *      This routine initiates the creation of an EVMS volume using the
  *      handle of a suitable storage object. In this case the user_data
  *      is the handle.
- * 
+ *
  *   Entry:
  *      menuitem  - the popup menuitem that was selected
  *      user_data - the user data which is the object handle
@@ -310,7 +310,7 @@ void on_create_volume_popup_menu_item_activate(GtkMenuItem * menuitem, gpointer
  *      This routine initiates the creation of a compatibility volume.
  *      The handle of the object to place the volume on top of is
  *      provided through the user data.
- * 
+ *
  *   Entry:
  *      menuitem  - the popup menuitem that was selected
  *      user_data - address of user data bound with signal (not used)
@@ -342,7 +342,7 @@ void on_create_compatibility_volume_popup_menu_item_activate(GtkMenuItem * menui
  *      This routine simply calls the function that handles
  *      displaying expand/shrink points for the selected
  *      volume.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - contains id of resize action EVMS_Task_Expand or
@@ -360,12 +360,12 @@ void on_resize_volume_button_clicked(GtkButton * button, gpointer user_data)
 /*
  *
  *   void on_resize_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of volumes that can be expanded or shrunk depending on
  *      the action.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - contains id of resize action EVMS_Task_Expand
@@ -409,11 +409,11 @@ void on_resize_volume_clist_realize(GtkWidget * widget, gpointer user_data)
 /*
  *
  *   void on_modify_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of volumes that can have their properties changed.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - address of user data bound with signal (not used)
@@ -461,11 +461,11 @@ void on_modify_volume_clist_realize(GtkWidget * widget, gpointer user_data)
  *      entry field in the modify volume window has changed.
  *      If the entry is void of characters or only contains
  *      whitespace then we disable the Modify button. If it
- *      entry now has text then we ensure that the Modify 
+ *      entry now has text then we ensure that the Modify
  *      button is enabled.
  *
  *   Entry:
- *      editable  - address of GtkEditable descendant object that 
+ *      editable  - address of GtkEditable descendant object that
  *                  received changed signal
  *      user_data - address of user data bound to widget (not used)
  *
@@ -523,7 +523,7 @@ gchar *get_volume_name_text(GtkWidget * widget)
  *   Description:
  *      This routine initiates the modification of a volume's properties.
  *      It invokes evms_set_volume_name() to update the volume name.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - address of user data bound with signal (not used)
@@ -565,7 +565,7 @@ void on_modify_volume_button_clicked(GtkButton * button, gpointer user_data)
  *      This routine sets the initial name displayed in the volume
  *      name entry field to the current volume name for the given
  *      volume handle.
- * 
+ *
  *   Entry:
  *      entry     - the id of the volume name entry field
  *      user_data - contains the volume object handle
@@ -602,7 +602,7 @@ void on_volume_name_entry_realize(GtkEntry * entry, gpointer user_data)
  *      This routine invokes the routine to create the standard
  *      modify volume window connects additional signal handlers
  *      for the modify volume window widgets.
- * 
+ *
  *   Entry:
  *      volume - the volume object handle
  *
@@ -641,8 +641,8 @@ GtkWidget *build_modify_volume_window(object_handle_t volume)
  *   Description:
  *      This routine initiates the the modification of a volume's properties.
  *      It creates another window that allows modifying the properties of
- *      of the currently selected volume. 
- * 
+ *      of the currently selected volume.
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - address of user data bound with signal (not used)
@@ -706,7 +706,7 @@ void on_modify_volume_selection_button_clicked(GtkButton * button, gpointer user
  *
  *   Description:
  *      This routine displays help on the modify volume properties window.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - address of user data bound with signal (not used)
@@ -729,7 +729,7 @@ void on_modify_volume_help_button_clicked(GtkButton * button, gpointer user_data
  *
  *   Description:
  *      This routine initiates the modification of an EVMS volume's properties.
- * 
+ *
  *   Entry:
  *      menuitem  - the popup menuitem that was selected
  *      user_data - the user data which is the object handle
@@ -759,7 +759,7 @@ void on_modify_volume_popup_menu_item_activate(GtkMenuItem * menuitem, gpointer
  *   Description:
  *      This routine converts the flags bitmap to a readable string
  *      representing the flags that are enabled.
- * 
+ *
  *   Entry:
  *      flags - the volume flags bitmap
  *
@@ -776,8 +776,8 @@ gchar *get_volume_status_string(u_int32_t flags)
 
 	status_gstring = g_string_new("");
 
-	if (flags & VOLFLAG_DIRTY) {
-		status_gstring = g_string_append(status_gstring, _("Dirty"));
+	if (flags & (VOLFLAG_DIRTY | VOLFLAG_FEATURE_HEADER_DIRTY)) {
+		status_gstring = g_string_append(status_gstring, _("Modified"));
 	}
 
 	if (flags & VOLFLAG_NEW) {
@@ -822,13 +822,6 @@ gchar *get_volume_status_string(u_int32_t flags)
 		status_gstring = g_string_append(status_gstring, _("Fsck pending"));
 	}
 
-	if (flags & VOLFLAG_DEFRAG) {
-		if (*(status_gstring->str) != '\0')
-			status_gstring = g_string_append(status_gstring, ", ");
-
-		status_gstring = g_string_append(status_gstring, _("Defragment pending"));
-	}
-
 	if (*(status_gstring->str) == '\0')
 		status_gstring = g_string_append(status_gstring, _("OK"));
 
@@ -847,9 +840,9 @@ gchar *get_volume_status_string(u_int32_t flags)
  *      info structure and produces a fake extended_info_array_t
  *      from it. The extended info field for the FSIM will allow
  *      a "More" button to get created if an FSIM is associated
- *      with the volume in order to get file system related 
+ *      with the volume in order to get file system related
  *      extended info.
- * 
+ *
  *   Entry:
  *      volume        - generic info about the volume from the engine
  *      extended_info - address where to store pointer to our pseudo
@@ -999,7 +992,7 @@ void create_volume_pseudo_extended_info(logical_volume_info_t * volume,
  *
  *   Description:
  *      This routine initiates the modification of an EVMS volume's properties.
- * 
+ *
  *   Entry:
  *      menuitem  - the popup menuitem that was selected
  *      user_data - the user data which is the object handle
@@ -1020,13 +1013,13 @@ void free_extended_info(extended_info_array_t * extended_info)
 /*
  *
  *   void on_display_volume_details_menu_item_activate (GtkMenuItem *, gpointer)
- *   
+ *
  *   Description:
  *      This routine displays the generic volume information as though it
  *      was extended info. If the volume has an associated FSIM then the
  *      user can click on the more button to see file system related info
  *      on the volume.
- * 
+ *
  *   Entry:
  *      menuitem  - menu item selected
  *      user_data - contains handle of the volume we want detailed info on
@@ -1069,7 +1062,7 @@ void on_display_volume_details_menu_item_activate(GtkMenuItem * menuitem, gpoint
  *   Description:
  *      This routine initiates the conversion of a compatibility volume
  *      into a native EVMS volume.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - not used
@@ -1119,12 +1112,12 @@ void on_convert_to_evms_volume_button_clicked(GtkButton * button, gpointer user_
 /*
  *
  *   void on_convert_to_evms_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of compatibility volumes that can be converted to native
  *      EVMS volumes.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - address of user data bound with signal (not used)
@@ -1166,12 +1159,12 @@ void on_convert_to_evms_volume_clist_realize(GtkWidget * widget, gpointer user_d
 /*
  *
  *   void on_convert_to_compatibility_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of EVMS volumes that can be converted back to compatibility
  *      volumes.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - address of user data bound with signal (not used)
@@ -1213,18 +1206,18 @@ void on_convert_to_compatibility_volume_clist_realize(GtkWidget * widget, gpoint
 /*
  *
  *   GtkWidget *create_convert_to_evms_volume_window (GtkSignalFunc, gpointer)
- *   
+ *
  *   Description:
  *      This routine creates the dialog that allows converting a compatibility
- *      volume in the selection list to a native EVMS volume. 
- *  
+ *      volume in the selection list to a native EVMS volume.
+ *
  *      It is essentially the create volume selection dialog with the title and
  *      button labels changed along with a couple different signal handlers.
- * 
+ *
  *   Entry:
  *      clist_realize_sighandler - function that will populate list when realized
  *      user_data                - address of user data to bind with signal handlers
- *      
+ *
  *   Exit:
  *      Returns the id of the dialog window created.
  *
@@ -1245,7 +1238,7 @@ GtkWidget *create_convert_to_evms_volume_window(GtkSignalFunc clist_realize_sigh
 	    gtk_object_get_data(GTK_OBJECT(window), "selection_window_next_button_label");
 
 	/*
-	 * Retitle the window and change the next_button label from "Create" to "Convert"    
+	 * Retitle the window and change the next_button label from "Create" to "Convert"
 	 */
 
 	gtk_window_set_title(GTK_WINDOW(window), _("Convert Compatibility Volume to EVMS Volume"));
@@ -1281,7 +1274,7 @@ GtkWidget *create_convert_to_evms_volume_window(GtkSignalFunc clist_realize_sigh
  *   Description:
  *      This routine initiates the conversion of a compatibility volume
  *      to an EVMS volume for one specific volume.
- * 
+ *
  *   Entry:
  *      menuitem  - the popup menuitem that was selected
  *      user_data - the user data which is the object handle
@@ -1306,7 +1299,7 @@ void on_convert_to_evms_volume_popup_menu_item_activate(GtkMenuItem * menuitem,
  *   Description:
  *      This routine initiates the conversion of an EVMS volume back to
  *      a compatibility volume.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - not used
@@ -1346,7 +1339,7 @@ void on_convert_to_compatibility_volume_button_clicked(GtkButton * button, gpoin
  *   Description:
  *      This routine initiates the conversion of an EVMS volume
  *      to a compatibility volume for one specific volume.
- * 
+ *
  *   Entry:
  *      menuitem  - the menuitem that initiated the action
  *      user_data - volume's object handle
@@ -1373,12 +1366,12 @@ void on_convert_to_compatibility_volume_popup_menu_item_activate(GtkMenuItem * m
 /*
  *
  *   void populate_clist_with_addible_features_for_volume (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of feature plug-ins that indicate can add a feature object
  *      to the given EVMS volume.
- * 
+ *
  *   Entry:
  *      clist     - address of the selections GtkCList widget
  *      user_data - volume's object handle
@@ -1431,7 +1424,7 @@ void populate_clist_with_addible_features_for_volume(GtkCList * clist, gpointer
  *      This routine retrieves the selected feature plug-in from the selection
  *      list and passes this along with any pre-selected volume handle to the
  *      routine that initiates the EVMS_Task_Add_Feature task.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - contains a volume handle
@@ -1453,7 +1446,7 @@ void on_add_feature_to_volume_button_clicked(GtkButton * button, gpointer user_d
  *      This routine will display the EVMS features that can add
  *      themselves between the the volume and the previous topmost
  *      object.
- * 
+ *
  *   Entry:
  *      menuitem  - the menuitem that initiated the action
  *      user_data - volume's object handle
@@ -1479,11 +1472,11 @@ void on_add_feature_to_volume_popup_menu_item_activate(GtkMenuItem * menuitem, g
 /*
  *
  *   void on_add_feature_to_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of volumes that can have a featured added to them.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - address of user data bound with signal (not used)
@@ -1528,10 +1521,10 @@ void on_add_feature_to_volume_clist_realize(GtkWidget * widget, gpointer user_da
  *
  *   Description:
  *      This routine retrieves the volume to add a feature to an creates
- *      the next window to select the feature. If the same volume was 
+ *      the next window to select the feature. If the same volume was
  *      previously selected and an existing feature list window exists, it
  *      it is re-displayed.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - the user data associated with the handler (not used)
@@ -1604,7 +1597,7 @@ void on_volume_to_add_feature_to_button_clicked(GtkButton * button, gpointer use
  *
  *   Description:
  *      This routine updates the mount options string.
- * 
+ *
  *   Entry:
  *      button       - address of the GtkButton widget
  *      mount_window - address of mount dialog window
@@ -1629,10 +1622,10 @@ void on_mount_options_button_clicked(GtkButton * button, GtkWidget * mount_windo
 
 /*
  *  GtkWidget *create_mount_options_window (GtkWidget *)
- * 
+ *
  *  Description:
  *     This routine creates a dialog window to be able to retrieve the mount command
- *     line options from the user. We modify an existing dialog with similar needs 
+ *     line options from the user. We modify an existing dialog with similar needs
  *     rather than make all the API calls to create one from scratch.
  */
 GtkWidget *create_mount_options_window(GtkWidget * mount_window)
@@ -1696,7 +1689,7 @@ GtkWidget *create_mount_options_window(GtkWidget * mount_window)
  *
  *   Description:
  *      This routine displays a dialog to set the mount options.
- * 
+ *
  *   Entry:
  *      button       - address of the GtkButton widget
  *      mount_window - address of mount dialog window
@@ -1717,18 +1710,18 @@ void on_set_mount_options_button_clicked(GtkButton * button, GtkWidget * mount_w
 /*
  *
  *   GtkWidget *create_mount_operation_window (GtkSignalFunc, gpointer)
- *   
+ *
  *   Description:
  *      This routine creates the dialog that allows for mounting a
- *      volume in the selection list to a native EVMS volume. 
- *  
+ *      volume in the selection list to a native EVMS volume.
+ *
  *      It is essentially the create volume selection dialog with the title and
  *      button labels changed along with a couple different signal handlers.
- * 
+ *
  *   Entry:
  *      clist_realize_sighandler - function that will populate list when realized
  *      user_data                - address of user data to bind with signal handlers
- *      
+ *
  *   Exit:
  *      Returns the id of the dialog window created.
  *
@@ -1837,7 +1830,7 @@ GtkWidget *create_mount_operation_window(gchar * title, gchar * verb,
  *
  *   Description:
  *      This routine initiates mounting a volume.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - object handle
@@ -1882,11 +1875,11 @@ void on_mount_volume_button_clicked(GtkButton * button, gpointer user_data)
 /*
  *
  *   void on_mount_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of volumes that can be mounted.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - not used
@@ -1906,7 +1899,7 @@ void on_mount_volume_clist_realize(GtkWidget * widget, gpointer user_data)
  *
  *   Description:
  *      This routine initiates mounting one volume.
- * 
+ *
  *   Entry:
  *      menuitem  - the menuitem that initiated the action
  *      user_data - object handle
@@ -1933,7 +1926,7 @@ void on_mount_volume_popup_menu_item_activate(GtkMenuItem * menuitem, gpointer u
  *
  *   Description:
  *      This routine initiates unmounting a volume.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - object handle
@@ -1967,7 +1960,7 @@ void on_unmount_volume_button_clicked(GtkButton * button, gpointer user_data)
  *
  *   Description:
  *      This routine initiates unmounting a volume.
- * 
+ *
  *   Entry:
  *      menuitem  - the menuitem that initiated the action
  *      user_data - object handle
@@ -1993,11 +1986,11 @@ void on_unmount_volume_popup_menu_item_activate(GtkMenuItem * menuitem, gpointer
 /*
  *
  *   void on_unmount_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of volumes that can be unmounted.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - not used
@@ -2017,7 +2010,7 @@ void on_unmount_volume_clist_realize(GtkWidget * widget, gpointer user_data)
  *
  *   Description:
  *      This routine initiates the remount of a volume.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - object handle
@@ -2036,7 +2029,7 @@ void on_remount_volume_button_clicked(GtkButton * button, gpointer user_data)
  *
  *   Description:
  *      This routine initiates re-mounting one volume.
- * 
+ *
  *   Entry:
  *      menuitem  - the menuitem that initiated the action
  *      user_data - object handle
@@ -2052,11 +2045,11 @@ void on_remount_volume_popup_menu_item_activate(GtkMenuItem * menuitem, gpointer
 /*
  *
  *   void on_remount_volume_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of volumes that can be re-mounted.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - not used
