@@ -26,8 +26,9 @@
 #include <getopt.h>
 #include <libgen.h>
 #include <frontend.h>
+#include <locale.h>
 
-#define REQUIRED_ENGINE_API_MAJOR 9
+#define REQUIRED_ENGINE_API_MAJOR 10
 #define REQUIRED_ENGINE_API_MINOR 0
 #define REQUIRED_ENGINE_API_PATCH 0
 
@@ -80,15 +81,15 @@ static struct query_commands_t {
 
 static int message_callback(char *message_text, int *answer, char **choices)
 {
-	LOG_VERBOSE(message_text);
-	LOG_VERBOSE("%s: Responding with default selection \"%s\".\n",
+	LOG_VERBOSE("%s", message_text);
+	LOG_VERBOSE(_("%s: Responding with default selection \"%s\".\n"),
 		    prog_name, choices[*answer]);
 	return 0;
 }
 
 static void status_callback(char *message_text)
 {
-	LOG_VERBOSE(message_text);
+	LOG_VERBOSE("%s", message_text);
 }
 
 static ui_callbacks_t callbacks = {
@@ -115,25 +116,25 @@ static boolean is_numeric(char * str)
 static void show_help(void)
 {
 	int i;
-	LOG("Usage: %s [options] <command> [command_options] [name]*\n", prog_name);
-	LOG("Display information about EVMS.\n"
-	    "   options:\n"
-	    "      [-d debug_level, --debug-level debug_level]\n"
-	    "         debug_level = [0-9] |\n"
-	    "                       [critical | serious | error | warning | default |\n"
-	    "                        details | entry_exit | debug | extra | everything]\n"
-	    "      [-l log_file_name, --log-file log_file_name]\n"
-	    "      [-n node_name, --node-name node_name]\n"
-	    "      [-v, --verbose] verbose mode\n"
-	    "      [-h | -? | --help] display this help\n"
-	    "   commands:\n");
+	LOG(_("Usage: %s [options] <command> [command_options] [name]*\n"), prog_name);
+	LOG(_("Display information about EVMS.\n"
+	      "   options:\n"
+	      "      [-d debug_level, --debug-level debug_level]\n"
+	      "         debug_level = [0-9] |\n"
+	      "                       [critical | serious | error | warning | default |\n"
+	      "                        details | entry_exit | debug | extra | everything]\n"
+	      "      [-l log_file_name, --log-file log_file_name]\n"
+	      "      [-n node_name, --node-name node_name]\n"
+	      "      [-v, --verbose] verbose mode\n"
+	      "      [-h | -? | --help] display this help\n"
+	      "   commands:\n"));
 	for (i = 0; i < NR_COMMANDS; i++) {
 		LOG("      %s\n", query_commands[i].name);
 	}
-	LOG("   command options:\n"
-	    "      [-i, --info]\n"
-	    "      [-a, --above]\n"
-	    "      [-b, --below]\n");
+	LOG(_("   command options:\n"
+	      "      [-i, --info]\n"
+	      "      [-a, --above]\n"
+	      "      [-b, --below]\n"));
 }
 
 static void parse_display_options(int argc, char **argv,
@@ -184,7 +185,7 @@ static int parse_options(int argc, char ** argv)
 
 				if ((level < CRITICAL) ||
 				    (level > EVERYTHING)) {
-					LOG("%s is not a valid debug level.\n",
+					LOG(_("%s is not a valid debug level.\n"),
 					    optarg);
 					/* Display the help. */
 					rc = 1;
@@ -210,7 +211,7 @@ static int parse_options(int argc, char ** argv)
 			} else if (strcasecmp(optarg, "everything") == 0) {
 				debug_level = EVERYTHING;
 			} else {
-				LOG("%s is not a valid debug level.\n",
+				LOG(_("%s is not a valid debug level.\n"),
 				    optarg);
 				/* Display the help. */
 				rc = EINVAL;
@@ -237,7 +238,7 @@ static int parse_options(int argc, char ** argv)
 			break;
 
 		default:
-			LOG("%s -- unrecognized option \"%c\"\n\n",
+			LOG(_("%s -- unrecognized option \"%c\"\n\n"),
 			    prog_name, c);
 			/* Display the help. */
 			rc = EINVAL;
@@ -262,7 +263,7 @@ static int get_command(char *command_name)
 		rc = strncmp(command_name, query_commands[i].name,
 			     strlen(query_commands[i].name));
 		if (!rc) {
-			LOG_VERBOSE("Found command: %s (%d)\n", command_name, i);
+			LOG_VERBOSE(_("Found command: %s (%d)\n"), command_name, i);
 			return i;
 		}
 	}
@@ -285,7 +286,7 @@ static int get_handle(char *name, engine_handle_t *handle)
 	if (rc) {
 		rc = evms_get_plugin_by_name(name, handle);
 		if (rc) {
-			LOG_ERROR("Error getting handle for \"%s\"\n", name);
+			LOG_ERROR(_("Error getting handle for \"%s\"\n"), name);
 		}
 	}
 
@@ -488,17 +489,17 @@ static void display_info_plugin(plugin_info_t *info, int name_only)
 	if (name_only) {
 		LOG("%s\n", info->short_name);
 	} else {
-		LOG("Name: %s\n", info->short_name);
-		LOG("Type: %s\n", object_type_string(PLUGIN));
-		LOG("Long Name: %s\n", info->long_name);
-		LOG("OEM Name: %s\n", info->oem_name);
-		LOG("Plugin ID: %d.%d.%d\n", GetPluginOEM(info->id),
+		LOG(_("Name: %s\n"), info->short_name);
+		LOG(_("Type: %s\n"), object_type_string(PLUGIN));
+		LOG(_("Long Name: %s\n"), info->long_name);
+		LOG(_("OEM Name: %s\n"), info->oem_name);
+		LOG(_("Plugin ID: %d.%d.%d\n"), GetPluginOEM(info->id),
 		    GetPluginType(info->id), GetPluginID(info->id));
-		LOG("Plugin Type: %s\n",
+		LOG(_("Plugin Type: %s\n"),
 		    plugin_type_string(GetPluginType(info->id)));
-		LOG("Plugin Version: %d.%d.%d\n", info->version.major,
+		LOG(_("Plugin Version: %d.%d.%d\n"), info->version.major,
 		    info->version.minor, info->version.patchlevel);
-		LOG("Required Engine Services API Version: %d.%d.%d\n",
+		LOG(_("Required Engine Services API Version: %d.%d.%d\n"),
 		    info->required_engine_api_version.major,
 		    info->required_engine_api_version.minor,
 		    info->required_engine_api_version.patchlevel);
@@ -518,17 +519,20 @@ static void display_info_object(storage_object_info_t *info, int name_only)
 	if (name_only) {
 		LOG("%s\n", info->name);
 	} else {
-		LOG("Name: %s\n", info->name);
-		LOG("Type: %s\n", object_type_string(info->object_type));
-		LOG("Data Type: %s\n", data_type_string(info->data_type));
-		LOG("Device Number: %d,%d\n",
+		char number_buffer[64];
+
+		LOG(_("Name: %s\n"), info->name);
+		LOG(_("Type: %s\n"), object_type_string(info->object_type));
+		LOG(_("Data Type: %s\n"), data_type_string(info->data_type));
+		LOG(_("Device Number: %d,%d\n"),
 		    info->dev_major, info->dev_minor);
-		LOG("Size (sectors): %"PRIu64"\n", info->size);
+		sprintf(number_buffer, "%"PRIu64, info->size);
+		LOG(_("Size (sectors): %s\n"), number_buffer);
 		rc = evms_get_info(info->plugin, &plugin_info);
 		if (rc) {
-			LOG("Plugin: Unknown\n");
+			LOG(_("Plugin: Unknown\n"));
 		} else {
-			LOG("Plugin: %s\n",
+			LOG(_("Plugin: %s\n"),
 			    plugin_info->info.plugin.short_name);
 			evms_free(plugin_info);
 		}
@@ -548,14 +552,17 @@ static void display_info_container(storage_container_info_t *info, int name_only
 	if (name_only) {
 		LOG("%s\n", info->name);
 	} else {
-		LOG("Name: %s\n", info->name);
-		LOG("Type: %s\n", object_type_string(CONTAINER));
-		LOG("Size (sectors): %"PRIu64"\n", info->size);
+		char number_buffer[64];
+
+		LOG(_("Name: %s\n"), info->name);
+		LOG(_("Type: %s\n"), object_type_string(CONTAINER));
+		sprintf(number_buffer, "%"PRIu64, info->size);
+		LOG(_("Size (sectors): %s\n"), number_buffer);
 		rc = evms_get_info(info->plugin, &plugin_info);
 		if (rc) {
-			LOG("Plugin: Unknown\n");
+			LOG(_("Plugin: Unknown\n"));
 		} else {
-			LOG("Plugin: %s\n",
+			LOG(_("Plugin: %s\n"),
 			    plugin_info->info.plugin.short_name);
 			evms_free(plugin_info);
 		}
@@ -575,16 +582,19 @@ static void display_info_volume(logical_volume_info_t *info, int name_only)
 	if (name_only) {
 		LOG("%s\n", info->name);
 	} else {
-		LOG("Name: %s\n", info->name);
-		LOG("Type: %s\n", object_type_string(VOLUME));
-		LOG("Device Number: %d:%d\n",
+		char number_buffer[64];
+
+		LOG(_("Name: %s\n"), info->name);
+		LOG(_("Type: %s\n"), object_type_string(VOLUME));
+		LOG(_("Device Number: %d:%d\n"),
 		    info->dev_major, info->dev_minor);
-		LOG("Size (sectors): %"PRIu64"\n", info->vol_size);
+		sprintf(number_buffer, "%"PRIu64, info->vol_size);
+		LOG(_("Size (sectors): %s\n"), number_buffer);
 		rc = evms_get_info(info->file_system_manager, &fsim_info);
 		if (rc) {
-			LOG("Filesystem: Unknown\n");
+			LOG(_("Filesystem: Unknown\n"));
 		} else {
-			LOG("Filesystem: %s\n",
+			LOG(_("Filesystem: %s\n"),
 			    fsim_info->info.plugin.short_name);
 			evms_free(fsim_info);
 		}
@@ -600,10 +610,10 @@ static void display_info_evms(void)
 {
 	evms_version_t engine_api;
 
-	LOG("EVMS Version: %d.%d.%d\n",
+	LOG(_("EVMS Version: %d.%d.%d\n"),
 	    MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL);
 	evms_get_api_version(&engine_api);
-	LOG("EVMS Engine API Version: %d.%d.%d\n",
+	LOG(_("EVMS Engine API Version: %d.%d.%d\n"),
 	    engine_api.major, engine_api.minor, engine_api.patchlevel);
 }
 
@@ -703,13 +713,13 @@ static void display_extended_info_value(extended_info_t *info)
 
 	switch (info->collection_type) {
 	case EVMS_Collection_None:
-		LOG("Value: ");
+		LOG(_("Value: "));
 		display_value(info->value, info->type);
 		LOG(" %s\n", value_unit_string(info->unit));
 		break;
 
 	case EVMS_Collection_List:
-		LOG("Value List:\n");
+		LOG(_("Value List:\n"));
 		for (i = 0; i < info->collection.list->count; i++) {
 			LOG("\t");
 			display_value(info->collection.list->value[i], info->type);
@@ -718,16 +728,16 @@ static void display_extended_info_value(extended_info_t *info)
 		break;
 
 	case EVMS_Collection_Range:
-		LOG("Value Range:\n");
-		LOG("\tMinimum: ");
+		LOG(_("Value Range:\n"));
+		LOG(_("\tMinimum: "));
 		display_value(info->collection.range->min, info->type);
 		LOG(" %s\n", value_unit_string(info->unit));
 
-		LOG("\tMaximum: ");
+		LOG(_("\tMaximum: "));
 		display_value(info->collection.range->max, info->type);
 		LOG(" %s\n", value_unit_string(info->unit));
 
-		LOG("\tIncrement: ");
+		LOG(_("\tIncrement: "));
 		display_value(info->collection.range->increment, info->type);
 		LOG(" %s\n", value_unit_string(info->unit));
 		break;
@@ -744,16 +754,16 @@ static void display_extended_info_value(extended_info_t *info)
 static void display_extended_info_item(extended_info_t *info)
 {
 	if (info->name) {
-		LOG("Name: %s", info->name);
+		LOG(_("Name: %s"), info->name);
 		if (info->flags & EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE) {
-			LOG(" (extra info available)");
+			LOG(_(" (extra information available)"));
 		}
 		LOG("\n");
 	}
 	if (info->title)
-		LOG("Title: %s\n", info->title);
+		LOG(_("Title: %s\n"), info->title);
 	if (info->desc)
-		LOG("Description: %s\n", info->desc);
+		LOG(_("Description: %s\n"), info->desc);
 
 	display_extended_info_value(info);
 }
@@ -812,13 +822,13 @@ static int display_handle(engine_handle_t handle, int name_only)
 
 	rc = evms_get_info(handle, &info);
 	if (rc) {
-		LOG_ERROR("Error getting info for handle 0x%x\n", handle);
+		LOG_ERROR(_("Error getting info for handle 0x%x\n"), handle);
 		goto out;
 	}
 
 	rc = display_info(info, name_only);
 	if (rc) {
-		LOG_ERROR("Error displaying info for handle 0x%x\n", handle);
+		LOG_ERROR(_("Error displaying info for handle 0x%x\n"), handle);
 		goto out;
 	}
 
@@ -893,7 +903,7 @@ static int display_plugin(engine_handle_t handle, int name_only)
 
 	rc = evms_get_info(handle, &info);
 	if (rc) {
-		LOG_ERROR("Error getting info for handle 0x%x\n", handle);
+		LOG_ERROR(_("Error getting info for handle 0x%x\n"), handle);
 		goto out;
 	}
 
@@ -940,7 +950,7 @@ static int display_children(engine_handle_t handle,
 
 	rc = evms_get_info(handle, &info);
 	if (rc) {
-		LOG_ERROR("Error getting info for handle 0x%x\n", handle);
+		LOG_ERROR(_("Error getting info for handle 0x%x\n"), handle);
 		goto out;
 	}
 
@@ -1005,7 +1015,7 @@ static int display_parents(engine_handle_t handle,
 
 	rc = evms_get_info(handle, &info);
 	if (rc) {
-		LOG_ERROR("Error getting info for handle 0x%x\n", handle);
+		LOG_ERROR(_("Error getting info for handle 0x%x\n"), handle);
 		goto out;
 	}
 
@@ -1068,7 +1078,7 @@ static int walk_tree(char *source_name, object_type_t type,
 
 	rc = evms_get_handle_object_type(source_handle, &source_type);
 	if (rc) {
-		LOG_ERROR("Error getting type for \"%s\"\n", source_name);
+		LOG_ERROR(_("Error getting type for \"%s\"\n"), source_name);
 		goto out;
 	}
 
@@ -1245,13 +1255,13 @@ static int details(object_type_t type, int argc, char **argv)
 
 		rc = evms_get_extended_info(handle, descriptor, &info);
 		if (rc) {
-			LOG_ERROR("Error getting details for \"%s\"\n", name);
+			LOG_ERROR(_("Error getting details for \"%s\"\n"), name);
 			goto out;
 		}
 
 		rc = display_extended_info(info);
 		if (rc) {
-			LOG_ERROR("Error displaying details for \"%s\"\n", name);
+			LOG_ERROR(_("Error displaying details for \"%s\"\n"), name);
 		}
 
 		evms_free(info);
@@ -1270,6 +1280,10 @@ int main(int argc, char **argv)
 	int command;
 	int rc = 0;
 
+	setlocale(LC_MESSAGES, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
 	prog_name = basename(argv[0]);
 
 	rc = parse_options(argc, argv);
@@ -1287,12 +1301,12 @@ int main(int argc, char **argv)
 
 	if ((engine_api.major != REQUIRED_ENGINE_API_MAJOR) ||
 	    (engine_api.major == REQUIRED_ENGINE_API_MAJOR &&
-	     engine_api.minor <  REQUIRED_ENGINE_API_PATCH)) {
-		LOG_ERROR("EVMS API version mismatch.\n");
-		LOG_ERROR("\tEVMS provides %d.%d.%d\n",
+	     engine_api.minor <  REQUIRED_ENGINE_API_MINOR)) {
+		LOG_ERROR(_("EVMS API version mismatch.\n"));
+		LOG_ERROR(_("\tEVMS provides %d.%d.%d\n"),
 			  engine_api.major, engine_api.minor,
 			  engine_api.patchlevel);
-		LOG_ERROR("\t%s requires %d.%d.%d\n",
+		LOG_ERROR(_("\t%s requires %d.%d.%d\n"),
 			  prog_name, REQUIRED_ENGINE_API_MAJOR,
 			  REQUIRED_ENGINE_API_MINOR, REQUIRED_ENGINE_API_PATCH);
 		return EPERM;
@@ -1307,7 +1321,7 @@ int main(int argc, char **argv)
 	rc = evms_open_engine(node_name, ENGINE_READWRITE,
 			      &callbacks, debug_level, log_file);
 	if (rc) {
-		LOG_ERROR("evms_open_engine() failed with error code %d: %s\n",
+		LOG_ERROR(_("evms_open_engine() failed with error code %d: %s\n"),
 			  rc, evms_strerror(rc));
 		return rc;
 	}
@@ -1315,7 +1329,7 @@ int main(int argc, char **argv)
 	rc = query_commands[command].fn(query_commands[command].type,
 					argc - optind, argv + optind);
 	if (rc) {
-		LOG_ERROR("\"%s\" command failed with error code %d: %s\n",
+		LOG_ERROR(_("\"%s\" command failed with error code %d: %s\n"),
 			  query_commands[command].name, rc, evms_strerror(rc));
 	}
 
